;compiler: avrasm
;************************************************************************************************
; MOD player 2
; MCU: ATmega32-16
; XTAL: 20MHz (don't remember CKOPT fuse!) or you can connect external XTAL oscillator
; LCD: HD44780 2x20
;
; Author: Stanislav Maslan, s.maslan@seznam.cz
; Description: this player can play 4 channel MOD files with 15 or 31 instruments
;              sampling rate is about 22321Hz, 16-bit stereo
;              output is I2S or "Japanese standard" 
;************************************************************************************************

	.include "m32def.inc"
	.dseg
	;variables
IDEVars:	.byte	1	;bitove promenne pro IDE zarizeni
;	.equ	IDEconB=0	;pritomnost IDE zarizeni
;	.equ	NonMagneticB=7	;nemagneticke medium
BitVars:	.byte	1	;bitove promenne
;	.equ	LdNewSongB=0	;nacti novou skladbu
;	.equ	SeveSongB=1	;uloz skladbu
;	.equ	EraseSongB=2	;smaz skladbu
;	.equ	ChangePatternB=3	;indikace rucni zmeny listu
;	.equ	DispInfoPauseB=4	;zobrazeni infa pri pauze
;	.equ	DispInfoPauseS=5	;neco jako bit 4, ale uz nevim, co to ma presne delat :-), PS.: musi to tam byt
;	.equ	SongSourceB=6	;1 - RAM, 0 - CF/HDD
;	.equ	PausePlayB=7	;1 - prehravani, 0- pauza
SongLen:	.byte	2	;delka skladby ve ctvericich sektoru
MemSize:	.byte	2	;velikost pameti ve ctvericich sektoru
ButtonStack:	.byte	3	;promenne pro tlacitka
BarGraf:	.byte	4	;promenne pro bargrafy (4 kanaly)
ArpeggioStack:	.byte	28	;promenne pro efekt arpeggio
VolSlideStack:	.byte	8	;promenne pro efekt volume slide
PortamentoStack:	.byte	8	;promenne pro efekty portamento
LinePTR:	.byte	3	;ukazatel na aktualni radek v listu (24b)
PatternList:	.byte	128	;zasobnik s poradim listu
PatternPos:	.byte	1	;cislo radku v listu (63-N)
PatternPTR:	.byte	1	;cislo perhravaneho listu
PatternN:	.byte	1	;pocet listu
HighestPatN:	.byte	1	;cislo nejvyssiho listu (jen pro urceni pozice vzorku)
PatIncSpeed:	.byte	10	;delitele pro casovani cteni "not" a efektu
SongSpeed:	.byte	1	;rychlost songu (standardne 6)
SmplsAdress:	.byte	310	;info o kazdem z 31 nebo 15 vzorku v souboru
PlayDataS:	.byte	76	;zasobnik obsahujici informace o prave prehravanem samplu pro kazdy kanal
SongPTR:	.byte	1	;cislo songu v externi pameti
SongN:	.byte	1	;pocet nalezenych skladeb
PatternStart:	.byte	2	;zacatek "not" (lisi pro ruzne druhy MODu)
PatList:	.byte	2	;zacatek ople listu v souboru (lisi pro ruzne druhy MODu)
SMPLsoftStart:	.byte	12
FIFOptr:	.byte	4	;ukazatele WR/RD e FIFO zasobniku
	.equ	FIFOsize=1200	;delka = FIFOsize/(4*22.321) [ms]
OutputFIFO:	.byte	FIFOsize
	;FIFOsize > 1028B and it have to be multiple of 4 (1028, 1032, 1036)!!!
	
			
	.cseg
	.org	$000	;init
	jmp	Init
	.org	$008	;timer 2 - button read
	jmp	CteniTlacitek
	.org	$00E	;timer 1 - timeout for data reception
	jmp	RecData
;************************************************************************************************
; this routine takes one sample for each channel and send it to the D/A
; sometimes in/out is used instead push/pop - it's faster 
;************************************************************************************************
	.org	$014	;timer 0 INT for FIFO output
	;tento podprogram musi byt co nejkratsi, takze v dusledku optimalizace dost neprehledny :-)
	;!!! push/pop je nahrazeno in/out do nevyuzitych registru casovace 1 a jinde se s nimi nesmi manipulovat (usetri se 4 cykly :-)!!!
	out	ocr1bh,R16	;misto push
	out	ocr1bl,R30	;misto push
	in	R16,sreg	;misto push
	out	icr1h,R31	;misto push
	out	icr1l,R16	;misto push

	lds	R30,FIFOptr	;nacteni RD adresy
	lds	R31,FIFOptr+1
	;tady je treba vybrat cast kodu pro TDA1543(I2S) nebo TDA1545("japonsky standard" casove multiplexovany, doplnkovy tvar (standard jak noha - prakticky jde o I2S, kde je signal WS zpozden o 1 hodinovy cykl))
	;here you have to select TDA1543 od TDA1545 (other buses)

;--------------------------------------------------------------------------------------------------------
;	this is output for DAC TDA1545
;	PS.: nic nemazat - je to presne nacasovano a kratsi uz to byt nemuze
;--------------------------------------------------------------------------------------------------------
	ld	R16,Z+
	out	spdr,R16	;high byte levy
	;15 cyklu pauza
	ldi	R16,5
ISPdel1:	dec	R16
	brne	ISPDel1
	;
	ld	R16,Z+
	out	spdr,R16	;low byte levy
	;pauza 15 cyklu
	ldi	R16,4
ISPdel2:	dec	R16
	brne	ISPdel2
	cpi	R30,low(OutputFIFO+FIFOsize-2)
	ldi	R16,high(OutputFIFO+FIFOsize-2)
	cpc	R31,R16
	;
	ld	R16,Z+
	sbi	portb,2	;negovat WS pin
	out	spdr,R16	;high byte pravy
	ld	R16,Z+
	;pauza 15 cyklu
	brne	NeKonec
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	rjmp	JoKonec
NeKonec:	nop
	nop
	nop
JoKOnec:	sts	FIFOptr,R30
	sts	FIFOptr+1,R31
	adiw	R30,0	;2 cykly
	adiw	R30,0	;2 cykly
	adiw	R30,0	;2 cykly
	;
	out	spdr,R16	;low byte pravy
	;pauza 15 cyklu
	adiw	R30,0	;2 cykly
	adiw	R30,0	;2 cykly
	adiw	R30,0	;2 cykly
	adiw	R30,0	;2 cykly
	adiw	R30,0	;2 cykly
	in	R31,icr1h	;misto pop
	in	R16,icr1l	;misto pop
	out	sreg,R16	;misto pop
	in	R16,ocr1bh	;misto pop
	in	R30,ocr1bl	;misto pop
	;
	cbi	portb,2	;negovat pin WS
	
;--------------------------------------------------------------------------------------------------------
;	output for I2S DAC TDA1543
;	PS.: nic nemazat - je to presne nacasovano a kratsi uz to byt nemuze
;--------------------------------------------------------------------------------------------------------
;	ld	R16,Z+
;	out	spdr,R16
;	;15 cyklu pauza
;	ldi	R16,5
;ISPdel1:	dec	R16
;	brne	ISPdel1
;	;
;	ld	R16,Z+
;	out	spdr,R16
;	;12 cyklu pauza
;	ldi	R16,3
;ISPdel2:	dec	R16
;	brne	ISPdel2
;	cpi	R30,low(OutputFIFO+FIFOsize-2)
;	ldi	R16,high(OutputFIFO+FIFOsize-2)
;	cpc	R31,R16
;	;
;	sbi	portb,2
;	nop
;	ld	R16,Z+
;	out	spdr,R16
;	ld	R16,Z+
;	;pauza 15 cyklu
;	brne	NeKonec
;	ldi	R30,low(OutputFIFO)
;	ldi	R31,high(OutputFIFO)
;	rjmp	JoKonec
;NeKonec:	nop
;	nop
;	nop
;JoKOnec:	sts	FIFOptr,R30
;	sts	FIFOptr+1,R31
;	adiw	R30,0	;2 cykly
;	adiw	R30,0	;2 cykly
;	adiw	R30,0	;2 cykly
;	;
;	out	spdr,R16
;	;pauza 15 cyklu
;	adiw	R30,0	;2 cykly
;	adiw	R30,0	;2 cykly
;	adiw	R30,0	;2 cykly
;	nop
;	in	R31,icr1h	;misto pop
;	in	R16,icr1l	;misto pop
;	out	sreg,R16	;misto pop
;	in	R16,ocr1bh	;misto pop
;	in	R30,ocr1bl	;misto pop
;	cbi	portb,2
	
	;sample is on the DAC
	reti
	
;*******************************************************************************************	
;	data reception timeout
;*******************************************************************************************
RecData:	ser	R25
	reti
	
;*******************************************************************************************	
;	reading buttons
;*******************************************************************************************
CteniTlacitek:	push	R20
	push	R19
	push	R17
	push	R16
	in	R16,sreg
	push	R16
	in	R16,ddra
	push	R16
	in	R16,porta
	push	R16
	;
	ldi	R16,0
	out	ddra,R16
	ldi	R16,255
	out	porta,R16
	nop
	nop
	nop
	nop

	call	RDbutony	;read states and compare with last states

	cpi	R20,0
	brne	NeNicStiskl
	rjmp	NicStiskl	;preskoc, kdyz nic neni stisknuto
	;obsluha pri stisku tlacitka (zmeny)
	;on button press
NeNicStiskl:	bst	R19,1	;song +
	brtc	NeIncSongZ
	call	IncSongPTR
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeIncSongZ:	bst	R19,0	;song -
	brtc	NeDecSongZ
	call	DecSongPTR
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeDecSongZ:	bst	R19,3	;next pattern
	brtc	NeIncListZ
	call	IncListPTR
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeIncListZ:	bst	R19,2	;previous pattern
	brtc	NeDecListZ
	call	DecListPTR
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeDecListZ:	bst	R19,4	;COM button
	brtc	NeLoadUSARTZ
	call	LoadSongUSART
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeLoadUSARTZ:	bst	R19,5	;START / STOP
	brtc	NeStartStopZ
	call	StartStop
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeStartStopZ:	bst	R19,6	;SAVE button
	brtc	NeSaveSongZ
	call	SaveSongB
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeSaveSongZ:	bst	R19,7	;ERASE button
	brtc	NeEraseSFZ
	call	EraseSongFor
	ldi	R16,1
	sts	ButtonStack+2,R16
	rjmp	NecoStiskl
NeEraseSFZ:	rjmp	StavySet	;-)

	;obsluha pri drzeni tlacitka (stavy)
	;on holding button for longer time
StavySet:	lds	R16,ButtonStack+1
	dec	R16
	sts	ButtonStack+1,R16
	breq	ButAutoDelayY
	rjmp	NicStiskl
ButAutoDelayY:	inc	R16
	sts	ButtonStack+1,R16
	;uz zjistovat
	bst	R20,1
	brtc	NeIncSongS	;song +
	ldi	R17,12
	call	AutoSpeedR
	brtc	NeIncSongS
	call	IncSongPTR
	rjmp	NecoStisklS
NeIncSongS:	bst	R20,0
	brtc	NeDecSongS	;song - 
	ldi	R17,12
	call	AutoSpeedR
	brtc	NeDecSongS
	call	DecSongPTR
	rjmp	NecoStisklS
NeDecSongS:	bst	R20,3
	brtc	NeIncListS	;next pattern
	ldi	R17,3
	call	AutoSpeedR
	brtc	NeIncListS
	call	IncListPTR
	rjmp	NecoStisklS
NeIncListS:	bst	R20,2
	brtc	NeDecListS	;previous pattern
	ldi	R17,3
	call	AutoSpeedR
	brtc	NeDecListS
	call	DecListPTR
	rjmp	NecoStisklS
NeDecListS:	rjmp	NicStiskl

NecoStiskl:	ldi	R16,15
	sts	ButtonStack+1,R16

NecoStisklS:	lds	R16,BitVars
	bst	R16,7
	brts	SSkey
	bst	R16,5
	brts	SSkey
	ori	R16,0b00010000	;zajisti, ze pri pauze dojde k zobrazeni pri zmene listu/songu
SSkey:	andi	R16,0b11011111
	sts	BitVars,R16
NicStiskl:	;nastavit porta do norm. stavu
	cbi	ddrd,2
	;
	pop	R16
	out	porta,R16
	pop	R16
	out	ddra,R16
	pop	R16
	out	sreg,R16
	pop	R16
	pop	R17
	pop	R19
	pop	R20
	reti
	;tady uz jednotlive funkce tlacitek
	;here are the main functions for each button
	;song +
IncSongPTR:	lds	R16,IDEvars
	bst	R16,7
	brtc	ISPTRzero	;kdyz neni HDD/CF
	lds	R16,SongPTR
	inc	R16
	lds	R17,SongN
	cpi	R17,0
	breq	ISPTRzero	;kdyz je pamet prazdna
	cp	R16,R17
	brne	NullSongInc
	clr	R16
NullSongInc:	sts	SongPTR,R16
	lds	R16,BitVars
	andi	R16,0b10000000
	ori	R16,0b00000001
	sts	BitVars,R16
	ret
	;kdyz neni HDD/CF nebo je prazdny
	;HDD empty
ISPTRzero:	clr	R16
	sts	SongPTR,R16	
	lds	R16,BitVars
	andi	R16,0b10000000
	ori	R16,0b01000001
	sts	BitVars,R16
	ret
	;song -
DecSongPTR:	lds	R16,IDEvars
	bst	R16,7
	brtc	DSPTRzero	;kdyz neni HDD/CF
	lds	R16,SongN
	cpi	R16,0
	breq	DSPTRzero	;kdyz je pamet prazdna
	lds	R16,SongPTR
	dec	R16
	cpi	R16,255
	brne	NullSongDec
	lds	R16,SongN
	dec	R16
NullSongDec:	sts	SongPTR,R16
	lds	R16,BitVars
	andi	R16,0b10000000
	ori	R16,0b00000001
	sts	BitVars,R16
	ret
	;kdyz neni HDD/CF nebo je prazdny
	;HDD empty
DSPTRzero:	clr	R16
	sts	SongPTR,R16	
	lds	R16,BitVars
	andi	R16,0b10000000
	ori	R16,0b01000001
	sts	BitVars,R16
	ret
	;next pattern
IncListPTR:	ldi	R16,1	;pri prvnim pruchodu testem na dalsi list se nacte
	sts	PatternPos,R16
	lds	R16,PatIncSpeed+2
	lds	R17,PatIncSpeed+3
	subi	R16,1
	sbci	R17,0
	sts	PatIncSpeed,R16
	sts	PatIncSpeed+1,R17
	lds	R17,PatternPTR
	inc	R17
	lds	R16,PatternN
	cp	R16,R17
	brne	NeIncListMax
	dec	R17
NeIncListMax:	sts	PatternPTR,R17
	lds	R16,BitVars
	ori	R16,0b00001000
	sts	BitVars,R16
	ret
	;prev. pattern
DecListPTR:	ldi	R16,1
	sts	PatternPos,R16
	lds	R16,PatIncSpeed+2
	lds	R17,PatIncSpeed+3
	subi	R16,1
	sbci	R17,0
	sts	PatIncSpeed,R16
	sts	PatIncSpeed+1,R17
	lds	R17,PatternPTR
	dec	R17
	cpi	R17,255
	brne	NeDecListMax
	ldi	R17,0
NeDecListMax:	sts	PatternPTR,R17
	lds	R16,BitVars
	ori	R16,0b00001000
	sts	BitVars,R16
	ret
	;nacist data z USARTu
	;load data from USART
LoadSongUSART:	lds	R16,BitVars
	andi	R16,0b10000000
	ori	R16,0b01000001
	sts	BitVars,R16
	ret
	;start/stop
StartStop:	lds	R16,BitVars
	ldi	R17,0b10000000
	eor	R16,R17
	ori	R16,0b00100000
	sts	BitVars,R16
	ret
	;ulozit skladbu
	;SAVE
SaveSongB:	lds	R16,IDEvars
	bst	R16,7
	brtc	NeSaveSongB	;pokud neni HDD/CF
	lds	R16,BitVars
	ori	R16,0b00000011	;nastavit priznak nove skladby a zaroven ulozeni dat
	sts	BitVars,R16
NeSaveSongB:	ret
	;mazat skladbu / formatovat
	;ERASE
EraseSongFor:	lds	R16,IDEvars
	bst	R16,7
	brtc	NeEraseSongFor	;pokud neni HDD/CF
	lds	R16,BitVars
	ori	R16,0b00000101	;nastavit priznak nove skladby a zaroven mazani dat
	sts	BitVars,R16
NeEraseSongFor:	ret
	;rychlost automatickeho pricitani pri drzeni
AutoSpeedR:	clt
	lds	R16,ButtonStack+2
	dec	R16
	sts	ButtonStack+2,R16
	brne	NeAutoSpeed
	sts	ButtonStack+2,R17
	set
NeAutoSpeed:	ret
;--------------------------------------------------------------------------------------------------------
;	nacte zmeny tlacitek do R19 a stavy do R20
;	this reads the button states and compare it with last reading
;	R19 - button press flags
;	R20 - button hold flags 
;--------------------------------------------------------------------------------------------------------
RDbutony:	sbi	ddrd,2
	nop
	nop
	nop
	nop
	in	R20,pina	;nacteni stavu na porta
	cbi	ddrd,2
	;vyhodnoceni zmen
	lds	R16,ButtonStack
	sts	ButtonStack,R20
	eor	R16,R20
	com	R20	;v R20 stavy (pri podrzeni tlacitka) jednotlivych tlacitek
	and	R16,R20
	mov	R19,R16	;v R19 zmeny (z nestiknuteho do stisknuteho stavu) jednotlivych tlacitek
	ret
	;cekat na stisk cudlu
WaitButPressed:	ldi	R16,2
	call	DelayLongVar
	ldi	R16,0b00000000
	out	ddra,R16
	call	RDbutony
	ldi	R16,0b11111111
	out	ddra,R16
	cpi	R19,0
	breq	WaitButPressed
	ret
	
;*******************************************************************************************
;	Initialization
;*******************************************************************************************
Init:	;pocatecni nastaveni portu a pinu
	;ports/pins initialization
	ldi	R16,0b11111111
	out	ddra,R16
	ldi	R16,0b11111011
	out	ddrd,R16
	ldi	R16,0b11111111
	out	ddrb,R16
	ldi	R16,0b00000000
	out	ddrc,R16
	sbi	portd,7	;IOWR
	sbi	portd,3	;IORD
	sbi	portb,3	;RAM WR
	sbi	portd,4	;RAM OE
	sbi	portd,6	;A16-A19 buffer CK
	sbi	portd,5	;A8-A15 buffer CK
	cbi	portb,0	;LCD E
	cbi	portb,1	;LCD RS
	cbi	portd,2	;buttons common pin
	;stack ptr
	ldi	R16,high(ramend)
	out	sph,R16	;nastav SP na konec SRAM
	ldi	R16,low(ramend)
	out	spl,R16	;nastav SP na konec SRAM
	;FIFO out timer 0
	ldi	R16,0b00001010	;timer 0: 22321,42 Hz
	out	tccr0,R16
	ldi	R16,111	;timer 0: 22321,42 Hz (111 for 20MHz)
	out	ocr0,R16
	;timer 2 for buttons
	ldi	R16,0b01000111
	out	tccr2,R16
	ldi	R16,255	;timer 2: asi 75Hz
	out	ocr2,R16
	;timeout timer 1
	ldi	R16,0b00000000
	out	tccr1a,R16
	ldi	R16,0b00001101	;delitel 1024
	out	tccr1b,R16
	;SPI rozhrani pro vytvoreni I2S sbernice
	;SPI for I2S emulation	
	ldi	R16,0b01010000
	out	spcr,R16
	ldi	R16,0b00000001
	out	spsr,R16
	;USART pro prijem dat, 8bit, 1 stop, bez parity
	ldi	R16,0b01100010
	out	ucsra,R16
	ldi	R16,0b00011000
	out	ucsrb,R16
	ldi	R16,0b10000110
	out	ucsrc,R16
	ldi	R16,0b00000000
	out	ubrrh,R16
	;this values are valid only for 20MHz XTAL
;	ldi	R16,21	;115200bd
;	ldi	R16,10	;230400bd
;	ldi	R16,4	;500000bd
;	ldi	R16,2	;833333bd
	ldi	R16,1	;1250000bd
;	ldi	R16,0	;2500000bd
	out	ubrrl,R16
	;pro zjistovani zmen tlacitek
	ser	R16	;pocatecni stav
	sts	ButtonStack,R16
	;init. LCD
	call	LCDinit	;init LCD jako 2 radkovy
	;identify IDE device
	call	IdentIDEdrive	;read ident. sector
	call	LCDwrInitInf	;LCD write info
	;zdroj dat (RAM/externi pamet)
	ldi	R16,0b11000000	;po zapnuti cist z RAM
	lds	R17,SongN
	cpi	R17,0	;kdyz je prazdny HDD/CF tak cist z RAM
	breq	PZCZR
	lds	R17,IDEvars
	bst	R17,7
	brtc	PZCZR
	ldi	R16,0b10000000	;kdyz je HDD/CF, cist z nej
PZCZR:	sts	BitVars,R16
	;
	ldi	R16,0	;ukazatel na 1. song v externi pameti
	sts	SongPTR,R16
	;vymazat delku skladby (aby se nedalo ukladat v pripade nacetni chybnych dat nebo skladby z CF/HDD)
	clr	R16
	sts	SongLen+1,R16
	sts	SongLen+0,R16
	;
LdNewSong:	call	NewSong	;init new song

;***************************************************************************************************	
; v nasledujicim kodu se generuje vlastni zvuk (nacitani vzorku z RAM a provadeni efektu) 
; vysledek se uklada do FIFO na adresu WR v zasobniku FIFOptr
;
; In this code the MOD is decoded. Output samples are stored into FIFO.    
;***************************************************************************************************
	;tady se resi START/STOP prehravani
	;this is used for START/STOP handling
CountSMPL:	cli
	lds	R16,BitVars
	in	R17,timsk
	bst	R16,7
	bld	R17,1
	out	timsk,R17
	bst	R16,4
	brtc	NeStopChange
	lds	R16,BitVars
	andi	R16,0b11101111
	sts	BitVars,R16
	bst	R16,0
	brtc	NeNewStop
	call	NewSong
NeNewStop:	call	LCDwriteStatus
	cli
	;tady se zjistuje kolik mista je ve FIFO
	;first I have to check free memory in FIFO, at least 8B for new sample 
NeStopChange:	lds	R30,FIFOptr	;read offset
	lds	R31,FIFOptr+1
	lds	R28,FIFOptr+2	;write offset
	lds	R29,FIFOptr+3
	sub	R30,R28
	sbc	R31,R29
	brcc	JeTamMisto
	ldi	R26,low(FIFOsize-1)
	ldi	R27,high(FIFOsize-1)
	add	R30,R26
	adc	R31,R27
JeTamMisto:	cpi	R30,8	;kdyz je alespon 8 bytu mista, povoli se vypocet dalsicho vzorku
	clr	R16
	cpc	R31,R16
	brcc	NeDohnal
	rjmp	UzDohnal	;nepocitat vzorek
NeDohnal:	sei
;--------------------------------------------------------------------------------------------------------	
;	there are 8 free Bytes or more, so we can generate sample
;--------------------------------------------------------------------------------------------------------
	;nejdriv se deli vzorkovaci frekvecne 8x - nasledujici instrukce pak v prumeru zaberou 1/8 casu 
	lds	R16,PatIncSpeed+5
	inc	R16
	sts	PatIncSpeed+5,R16
	cpi	R16,8
	breq	DalsiRadekNot
	rjmp	NeDalsiRadekNot	;7 z 8 pruchodu se preskakuji dalsi vypocty az k vlastnimu vypoctu vzorku
DalsiRadekNot:	clr	R16
	sts	PatIncSpeed+5,R16
;--------------------------------------------------------------------------------------------------------
;	this code is processed each 8 samples (for speed increase)
	;toto deleni 8x ma stejnou funkci jako predchozi - usetrit cas, tentokdat instrukce zabiraji jen 1/64 casu
	lds	R16,PatIncSpeed+6
	inc	R16
	sts	PatIncSpeed+6,R16
	cpi	R16,8
	breq	CanDoEfekty
	rjmp	NotCanDoEfekty	;7 z 8 pruchodu se preskakuji vsechny dalsi instrukce az k testu, kde se zjistuje jesl se ma cist dalsi radek not 
CanDoEfekty:	clr	R16
	sts	PatIncSpeed+6,R16
;--------------------------------------------------------------------------------------------------------
;	this code is processed each 64 samples (for speed increase)
	;test, jestli se ma nacist dali song
	;test for new song (button pressed)
	lds	R16,BitVars
	bst	R16,0
	brtc	NeJinejSong
	cli
	rjmp	LdNewSong	;next song
NeJinejSong:	;dalsi delitel, pro vykresleni bargrafu (kresli se asi 70x/s)
	;and this is next divider (only 70x/sec)
	lds	R16,PatIncSpeed+7
	inc	R16
	sts	PatIncSpeed+7,R16
	cpi	R16,5
	brne	NeBarGs
	clr	R16
	sts	PatIncSpeed+7,R16
	;rewrite LCD VU
	call	WriteBarGraf
NeBarGs:	;effect volume slide (350x/s)
	ldi	R30,low(VolSlideStack)
	ldi	R31,high(VolSlideStack)	
	ldi	R28,low(PlayDataS+7)
	ldi	R29,high(PlayDataS+7)
	ldi	R25,4	;for all 4ch
	
SlideVolumeC:	ld	R0,Y+
	ld	R1,Y+
	sbiw	R28,2
	ld	R16,Z+
	ld	R17,Z+
	bst	R17,7
	brts	VolumeSlideUp
	;slide down
	sub	R0,R16
	sbc	R1,R17
	brcc	NeVSlideDownZ
	clr	R0
	clr	R1
NeVSlideDownZ:	rjmp	VolumeSlideOk
VolumeSlideUp:	;slide up
	andi	R17,0b01111111
	add	R0,R16
	adc	R1,R17
	brcc	VolumeSlideOk
	clr	R0
	ser	R16
	mov	R1,R16
VolumeSlideOk:	st	Y+,R0
	st	Y+,R1
	adiw	R28,17
	dec	R25
	brne	SlideVolumeC
	
	;dalsi delitel pro efekt arpeggio (asi 50x/s)
	;this is next divider for arpeggio effect (50x/sec) 
	lds	R16,PatIncSpeed+8
	inc	R16
	sts	PatIncSpeed+8,R16
	cpi	R16,7
	brne	NeArpeggioDiv
	ldi	R16,0
	sts	PatIncSpeed+8,R16
	;arpeggio
	ldi	R30,low(ArpeggioStack)
	ldi	R31,high(ArpeggioStack)	
	ldi	R28,low(PlayDataS+3)
	ldi	R29,high(PlayDataS+3)
	ldi	R25,4	;for all 4ch
	
NextChanArp:	ld	R20,Z+
	cpi	R20,0
	breq	NedelatArpeggio	;neaktivni arpeggio
	;vyber mezi zakladnim, 1. a 2. kmitoctem arpeggia
	;select 1. or 2. period of this effect
	cpi	R20,1
	brne	NeZaklArp1
	ld	R16,Z+
	ld	R17,Z+
	clr	R18
	st	Y+,R18
	st	Y+,R16
	st	Y+,R17
	sbiw	R28,3
	sbiw	R30,3
	ldi	R20,2
	st	Z+,R20
	rjmp	NedelatArpeggio
NeZaklArp1:	cpi	R20,2
	brne	NeZaklArp2
	adiw	R30,2
	ld	R16,Z+
	ld	R17,Z+
	clr	R18
	st	Y+,R18
	st	Y+,R16
	st	Y+,R17
	sbiw	R28,3
	sbiw	R30,5
	ldi	R20,3
	st	Z+,R20
	rjmp	NedelatArpeggio
NeZaklArp2:	cpi	R20,3
	brne	NedelatArpeggio
	adiw	R30,4
	ld	R16,Z+
	ld	R17,Z+
	clr	R18
	st	Y+,R18
	st	Y+,R16
	st	Y+,R17
	sbiw	R28,3
	sbiw	R30,7
	ldi	R20,1
	st	Z+,R20
NedelatArpeggio:	adiw	R30,6
	adiw	R28,19
	dec	R25
	brne	NextChanArp
	
	;dalsi deleni, pro portamento (asi 32x/s)
	;this is divider for portamento effect
NeArpeggioDiv:	lds	R16,PatIncSpeed+9
	inc	R16
	sts	PatIncSpeed+9,R16
	cpi	R16,10
	breq	PortamentoDS
	rjmp	NePortamentoDS
PortamentoDS:	clr	R16
	sts	PatIncSpeed+9,R16
	;pocitat portamento
	ldi	R30,low(PortamentoStack)
	ldi	R31,high(PortamentoStack)	
	ldi	R28,low(PlayDataS+3)
	ldi	R29,high(PlayDataS+3)
	ldi	R26,4	;for all 4ch
	
NextChanPort:	ld	R23,Z+	;nacist portam. konst.
	ld	R24,Z+
	mov	R25,R24
	ldi	R16,255	;16777215
	ldi	R17,255
	ldi	R18,255
	ld	R19,Y+
	ld	R20,Y+
	ld	R21,Y+	;current freq.
	bst	R24,5
	brtc	NePortTed	;kdyz neni portamento aktivni
	lsr	R21	;delit ji 2x (pro vyssi rozliseni po deleni)
	ror	R20
	ror	R19
	call	Div24u	;convert to the period

	bst	R24,7
	brts	PortUpCo	;now change period
	;odecitani
	andi	R24,0b00011111
	sub	R16,R23
	sbc	R17,R24
	sbci	R18,0
	rjmp	PortComCo
	;pricitani
PortUpCo:	andi	R24,0b00011111
	add	R16,R23
	adc	R17,R24
	clr	R23
	adc	R18,R23
PortComCo:	mov	R19,R16
	mov	R20,R17
	mov	R21,R18
	ldi	R16,255	;16777215
	ldi	R17,255
	ldi	R18,255
	call	Div24u	;and back to the freq.
	lsl	R16	;nasobit ji 2x
	rol	R17
	rol	R18
	;tone-portamento?
	bst	R25,6
	brtc	NeTonePortC
	;ano, test jestli bylo dosazeno cilove frekvence, kdyz ano, tak zastavit portamento
	;test for the end of portamento if it was tone-portamento effect
	sbiw	R28,5
	ld	R19,Y+
	ld	R20,Y+
	adiw	R28,3
	bst	R25,7
	brts	TPortCo
	;pri zvysovani
	cp	R17,R19
	cpc	R18,R20
	brcs	TportComCo
	clr	R16
	mov	R17,R19
	mov	R18,R20
	andi	R25,0b00011111	;zastavit tone portamento
	rjmp	TportComCo
TPortCo:	;pri snizovani
	cp	R19,R17
	cpc	R20,R18
	brcs	TportComCo
	clr	R16
	mov	R17,R19
	mov	R18,R20
	andi	R25,0b00011111	;zastavit tone portamento
TportComCo:	sbiw	R30,1
	st	Z+,R25
	;
NeTonePortC:	sbiw	R28,3
	st	Y+,R16	;ulozit zpet novou frekvenci
	st	Y+,R17
	st	Y+,R18
NePortTed:	adiw	R28,16

	dec	R26
	breq	NePortamentoDS
	rjmp	NextChanPort
NePortamentoDS:	;konec efektu

	;test jesli se ma cist dalsi radek not
	;test for next line of notes from pattern
NotCanDoEfekty:	cli
	lds	R30,PatIncSpeed
	lds	R31,PatIncSpeed+1
	adiw	R30,1
	sts	PatIncSpeed,R30
	sts	PatIncSpeed+1,R31
	lds	R18,PatIncSpeed+2
	lds	R19,PatIncSpeed+3
	cp	R30,R18
	cpc	R31,R19
	brne	NeDalsiRadekNot
	;dalsi noty
	;read next line
	clr	R16
	sts	PatIncSpeed,R16
	sts	PatIncSpeed+1,R16
	sei
	call	Takt	;nacist dalsi radek not
NeDalsiRadekNot:	sei
;********************************************************************************************
;	tady se podle dat v PlayDataS zasobniku nacitaji vlastni zvukova data 
;	a provadi se mix kanalu + par dalsich veci
;	nejdrive definice par promennych
;	PS.: jestli se nekomu zda program v teto casti nelogicky rozmisteny, 
;	zmateny apod., tak je to jen kvuli dosazeni maximalni rychlosti
;
;	here is the main sound generating routine (resampling, mixing, volume, ...)
;******************************************************************************************** 
	ldi	R26,low(SMPLsoftStart)
	ldi	R27,high(SMPLsoftStart)	;this is for instrument soft start
	ldi	R28,low(PlayDataS)
	ldi	R29,high(PlayDataS)	;actual play data buffer
	ldi	R30,low(BarGraf)
	ldi	R31,high(BarGraf)	;and this is for VUs
	ldi	R20,4	;4ch
	ldi	R21,0b10000000	;for signed byte conversion
	clr	R22	;0
	clr	R13
	clr	R14
	clr	R15	;left ch
	clr	R10
	clr	R11
	clr	R12	;right ch
NextChannelP:	;test for instrument softstart (elimination of starts and ends of instruments)
	ld	R16,X+
	bst	R16,7
	brtc	NeSoftStartSM
	;provadet soft start
	bst	R16,6
	brts	SoftStrtVolDown
	;uz volume up
	adiw	R28,4	;preskocit par nepotrebnych bytu
	ld	R16,Y+	;konstanta pro generovani tonu (frekvence)
	ld	R17,Y+
	adiw	R28,2	;zase par zde nepotrebnych bytu
	;pricist 50 k hlasitosti
	adiw	R26,1
	ld	R19,X	;nacist hlasitost
	ldi	R18,50
	add	R19,R18
	ld	R18,Y+	;hlasitost
	brcs	SoftVolUpC	;uz dosazeno pozadovane hlasitosti
	cp	R19,R18
	brcs	SoftVolUpNC
	;dosazeno pozadovane hlasitosti
SoftVolUpC:	mov	R19,R18
	ldi	R18,0b00000000	;soft start dokoncen
	sbiw	R26,2
	st	X+,R18	;vypnout soft start
	rjmp	SoftVolUp
	;jeste nedosazena max hlasitost
SoftVolUpNC:	st	X+,R19	;ulozit novou hlasitost
	sbiw	R26,2
	rjmp	SoftVolUp
	;teprve volume down - postupne snizeni hlasitosti minuleho nastroje
SoftStrtVolDown:	ld	R18,X+	;minuly vzorek (posledni z minuleho nastroje)
	ld	R19,X	;hlasitost
	subi	R19,50	;odecist 50 od hlasitosti
	brcc	SoftVolDownNC
	;uz je nulova (resp. ted bude)
	clr	R19
	bst	R16,5
	ldi	R16,0b10000000	;soft start - ale uz nabeh dalsiho vzorku
	brtc	NeJenVolDown
	ldi	R16,0b11100000	;drzet hlasitost v 0
NeJenVolDown:	sbiw	R26,2
	st	X+,R16
	adiw	R26,1	;pro dalsi kanal
	;neni nulova hlasitost
SoftVolDownNC:	st	X+,R19
	;tady se musi pricist konstanta tonu, jinak jsou kanaly rozfazovane
	adiw	R28,4	;preskocit par nepotrebnych bytu
	ld	R16,Y+	;konstanta pro generovani tonu (frekvence)
	ld	R17,Y+
	adiw	R28,3
	ld	R3,Y+	;pozice vzorku
	ld	R0,Y+
	ld	R1,Y+
	ld	R2,Y+
	add	R3,R16
	adc	R0,R17
	adc	R1,R22	;0
	adc	R2,R22	;0
	sbiw	R28,4
	st	Y+,R3
	st	Y+,R0
	st	Y+,R1
	st	Y+,R2
	adiw	R28,6
	;
	rjmp	SoftVolDown
	;here the resampling starts
NeSoftStartSM:	adiw	R28,4	;skip first 4 bytes
	ld	R16,Y+
	ld	R17,Y+	;resampling constant
	adiw	R28,2	;skip 2B
	ld	R19,Y+	;volume
	
SoftVolUp:	clr	R9	;some bit var
	
	ld	R3,Y+	;this is fixed point fraction 
	ld	R0,Y+
	ld	R1,Y+
	ld	R2,Y+	;actual position in instrument (24.8 fixed point)
	ld	R4,Y+
	ld	R5,Y+
	ld	R6,Y+	;end of instrument in RAM
	;add resampling const
	add	R3,R16
	adc	R0,R17
	adc	R1,R22	;0
	adc	R2,R22	;0
	;tests for end or loop
	cp	R0,R4	;zjistit, jestli uz je konec samplu
	cpc	R1,R5
	cpc	R2,R6
	brcs	NeKonecPl
	;uz jsem na konci
	ld	R4,Y+	;nahrat start loop vzorku
	ld	R5,Y+
	ld	R6,Y+
	bst	R6,7
	brtc	LoopSmpl	;ma se cyklit?
	;ne
	sbiw	R28,10	;navrat na pozici vzorku
	ldi	R16,0b11100000	;volume slide down
	sbiw	R26,1
	st	X+,R16
	rjmp	JoKonecPl
	;this was the last sample +1
LoopSmpl:	brne	LoopSmplNotLast	;last sample?
	;yes
	ldi	R16,255
	mov	R9,R16	;last sample flag
	rjmp	LoopSMPLlast
LoopSmplNotLast:	movw	R0,R4
	mov	R2,R6
LoopSMPLlast:	sbiw	R28,3
	;
NeKonecPl:	sbiw	R28,7
	;store new position into the buffer
JoKonecPl:	st	Y+,R3
	st	Y+,R0
	st	Y+,R1
	st	Y+,R2
	adiw	R28,6
	;load this sample
	out	porta,R2
	cbi	portd,5
	sbi	portd,5
	out	porta,R1
	cbi	portd,6
	sbi	portd,6
	out	porta,R0	;set RAM address
	;adresu na dalsi vzorek v RAM nebo mozna na start-loop adresu podle R9
	ldi	R17,1
	cp	R9,R22
	breq	NeLdLastSMPL
	;posledni vzorek, takze nacist start-loop adresu
	movw	R0,R4
	mov	R2,R6
	clr	R17	;a nepricitat 1
	;
NeLdLastSMPL:	add	R0,R17	;sample num ++
	adc	R1,R22	;0
	adc	R2,R22	;0
	;
	in	R18,pinc	;load 1.sample byte
	;poslat adresu 2. bytu ven
	out	porta,R2
	cbi	portd,5
	sbi	portd,5
	out	porta,R1
	cbi	portd,6
	sbi	portd,6
	out	porta,R0
	nop
	eor	R18,R21	;prevest 1.byte na neznaminkovy
	mov	R16,R18
	in	R17,pinc	;load next sample from this instrument
	eor	R17,R21	;prevest 2.byte na neznamenkovy
	
	;we have 2 samples now and the position between them (fraction of the sample position in R3)
	;so we can make linear interpolation
	cp	R17,R18
	brcs	Sestupna	;zjistis jestli jde o sest. nebo vzest. "hranu" signalu
	;rising edge (1.Q)
	sub	R17,R18
	mul	R17,R3
	add	R18,R1
	rjmp	Hotovo
	;faling edge (4.Q)
Sestupna:	sub	R16,R17
	mul	R16,R3
	sub	R18,R1
	;ulozit vypocitany vzorek do softstart zasobniku
	;store valueas into the softstart buffer
Hotovo:	st	X+,R18	;this is linear interpolated sample value
	st	X+,R19	;and this is actual volume
	;vypocet hlasitost (od ted bude singal v 16-bitech)
SoftVolDown:	eor	R18,R21	;to signed
	mulsu	R18,R19	;apply volume (just multiply) and we have the result in 16 bits
	eor	R1,R21	;to unsigned again
	;left/righ channel destiantion
	;vystup do mixeru tady se 4 kanaly mixuji na 2 
	;vyber levy/pravy
	cpi	R20,1
	breq	PravyK
	cpi	R20,4
	breq	PravyK
	;levy
	add	R13,R0
	adc	R14,R1
	adc	R15,R22	;0
	rjmp	PLkanOk
	;pravy
PravyK:	add	R10,R0
	adc	R11,R1
	adc	R12,R22	;0	
	;
PLkanOk:	ld	R16,Z	;store peak value for the VUs
	cp	R16,R1
	brcc	NeVicBars
	mov	R16,R1
NeVicBars:	st	Z+,R16
	;next channel
	dec	R20
	breq	NeNextChannelP
	rjmp	NextChannelP
NeNextChannelP:	;job complete, so just assign to the 16 bits
	lsr	R15
	ror	R14
	ror	R13
	lsr	R12
	ror	R11
	ror	R10
	eor	R14,R21	;a prevod do znamenkoveho tvaru
	eor	R11,R21	;a ro 2 kanal
	;test, zda uz je dosazeni konce FIFO pameti
	;now store the new samples to the FIFO
	cli
	lds	R28,FIFOptr+2	;na WR adresu
	lds	R29,FIFOptr+3
	st	Y+,R14	;save data byte levy
	st	Y+,R13
	st	Y+,R11	;save data byte pravy
	st	Y+,R10
	cpi	R28,low(OutputFIFO+FIFOsize)
	ldi	R16,high(OutputFIFO+FIFOsize)
	cpc	R29,R16
	brne	PresMaxW
	ldi	R28,low(OutputFIFO)	;navrat na zacatek
	ldi	R29,high(OutputFIFO)
PresMaxW:	sts	FIFOptr+2,R28
	sts	FIFOptr+3,R29
UzDohnal:	sei
	rjmp	CountSMPL	;pocitat dalsi
	
;********************************************************************************************
;	nacist do RAM novy song
;	new song initialization
;********************************************************************************************
NewSong:	;zastavit cteni cudlu a play timer
	in	R16,timsk
	andi	R16,0b01111101
	out	timsk,R16
	;ukazatele ve FIFO zasobniku
	ldi	R30,low(OutputFIFO+0)	
	ldi	R31,high(OutputFIFO+0)
	ldi	R28,low(FIFOptr)
	ldi	R29,high(FIFOptr)
	st	Y+,R30	;RD adresa ve FIFO
	st	Y+,R31
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	st	Y+,R30	;WR adresa ve FIFO
	st	Y+,R31
	;vypnout vsechny efekty
	clr	R16
	sts	PortamentoStack,R16
	sts	PortamentoStack+1,R16
	sts	PortamentoStack+2,R16
	sts	PortamentoStack+3,R16	
	sts	PortamentoStack+4,R16
	sts	PortamentoStack+5,R16
	sts	PortamentoStack+6,R16
	sts	PortamentoStack+7,R16
	sts	VolSlideStack,R16
	sts	VolSlideStack+1,R16
	sts	VolSlideStack+2,R16
	sts	VolSlideStack+3,R16
	sts	VolSlideStack+4,R16
	sts	VolSlideStack+5,R16
	sts	VolSlideStack+6,R16
	sts	VolSlideStack+7,R16
	sts	ArpeggioStack,R16
	sts	ArpeggioStack+7,R16
	sts	ArpeggioStack+14,R16
	sts	ArpeggioStack+21,R16
	;nulovat zasobnik pro soft-start nastroje (nevim jak jinak bych to nazval)
	sts	SMPLsoftstart+2,R16
	sts	SMPLsoftstart+5,R16
	sts	SMPLsoftstart+8,R16
	sts	SMPLsoftstart+11,R16
	ldi	R16,0b11100000	;pro vsechny kanaly volume slide down
	sts	SMPLsoftStart+0,R16
	sts	SMPLsoftStart+3,R16
	sts	SMPLsoftStart+6,R16
	sts	SMPLsoftStart+9,R16
	;konstanty pro speed
	ldi	R16,low(333)
	sts	PatIncSpeed,R16
	ldi	R16,high(333)
	sts	PatIncSpeed+1,R16
	ldi	R16,low(334)
	sts	PatIncSpeed+2,R16
	ldi	R16,high(334)
	sts	PatIncSpeed+3,R16
	;konstanty pro delitele
	ldi	R16,7
	sts	PatIncSpeed+5,R16	;/8 samplu
	ldi	R16,0
	sts	PatIncSpeed+6,R16	;/8/8 samplu
	ldi	R16,0
	sts	PatIncSpeed+7,R16	;pro bargrafy
	ldi	R16,0
	sts	PatIncSpeed+8,R16	;pro arpeggio
	ldi	R16,0
	sts	PatIncSpeed+9,R16	;pro portamento
	ldi	R16,125
	sts	PatIncSpeed+4,R16	;tempo 125
	ldi	R16,6
	sts	SongSpeed,R16	;speed 6
	;
	lds	R16,BitVars
	andi	R16,0b11000110	;nulovat vse mimo zdroj dat a priznaky zapisu/mazani
	sts	BitVars,R16
	;ukladat skladbu?
	bst	R16,1
	brtc	NeSaveSongIDE
	;ano
	call	SaveSongIDE
	rjmp	NewSongOk
NeSaveSongIDE:	;mazat skladbu/formatovat
	bst	R16,2
	brtc	NeEraseForZ
	;ano
	call	SongIDEeraseF
	rjmp	NewSongOk
NeEraseForZ:	;neprovadelo se mazani ani zapis, volba zdroje dat pro prehravani
	bst	R16,6
	brtc	NeRAMplay
	;hrat z RAM
	call	USARTload
	rjmp	NewSongOk
NeRAMplay:	;hrat z CF/HDD
	call	LoadStartAd
	;nulovat priznaky
NewSongOk:	lds	R16,BitVars
	andi	R16,0b11000000	;nulovat vse mimo zdroj dat
	sts	BitVars,R16
	;napsat nazev skladby na LCD
	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	clr	R0
	clr	R1
	clr	R2
	ldi	R19,20	;20 znaku nazvu
	;nacteni nazvu do RAM
PisNazev:	call	LoadData
	cpi	R16,0
	brne	NeKonecName
	ldi	R16,' '
NeKonecName:	inc	R0
	st	Z+,R16
	dec	R19
	brne	PisNazev
	;vypis na LCD
	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R25,20
	call	LCDwrStrRAM
	;
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4
	;nacist poradi listu + zjistit cislo nejvyssiho	
	call	LoadPatternList	;load pattern list
	;nacist info o vzorcich
	call	LoadSmplList	;load samples info
	;nastavit list
	call	SetPatternPTR	;select first list
	;vymazat play stack
	call	ClearPlayDS	;clear play stack
	;vymazat FIFO
	call	ClrFIFOgarbage	;clear FIFO
	;povolit cteni tlacitek a play timer 0
	ldi	R16,0b10000010
	out	tifr,R16
	in	R16,timsk
	ori	R16,0b10000010
	out	timsk,R16
	;
	sei
	ret
	;vymazat FIFO
ClrFIFOgarbage:	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R28,low(FIFOsize)
	ldi	R29,high(FIFOsize)
	ldi	R16,0
UnGarbageFIFO:	st	Z+,R16
	subi	R28,1
	sbci	R29,0
	brne	UnGarbageFIFO
	ret
	;vycisti zasobnik prehravanych dat
ClearPlayDS:	ldi	R30,low(PlayDataS)
	ldi	R31,high(PlayDataS)
	ldi	R20,76
	clr	R0
CPDS:	st	Z+,R0
	dec	R20
	brne	CPDS
	ret
	;this is for the VU generation
WriteBarGraf:	;blokovat tlacitka
	cli
	in	R16,timsk
	andi	R16,0b01111111
	out	timsk,R16
	sei
	;
	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16	
	ldi	R30,low(BarGraf)
	ldi	R31,high(BarGraf)
	ldi	R20,4	;4 kanaly
DalsiBarG:	ld	R16,Z+
	subi	R16,128
	swap	R16
	andi	R16,0b00000111
	call	LCDwrChar
	dec	R20
	brne	DalsiBarG
	;mix levy a pravy
	ldi	R16,' '
	call	LCDwrChar
	lds	R16,BarGraf
	subi	R16,128
	lds	R17,BarGraf+3
	subi	R17,128
	add	R16,R17
	swap	R16
	lsr	R16
	andi	R16,0b00000111
	call	LCDwrChar
	lds	R16,BarGraf+1
	subi	R16,128
	lds	R17,BarGraf+2
	subi	R17,128
	add	R16,R17
	swap	R16
	lsr	R16
	andi	R16,0b00000111
	call	LCDwrChar
	;fall speed - pomale klesani sloupcu :-)
	ldi	R30,low(BarGraf)
	ldi	R31,high(BarGraf)
	ldi	R20,4
NextChanBar1:	ld	R16,Z
	subi	R16,4	;fall speed (perioda)
	cpi	R16,128
	brcc	NextChanBar
	ldi	R16,128
NextChanBar:	st	Z+,R16
	dec	R20
	brne	NextChanBar1
	ldi	R16,0b11001101
	call	LCDsetPos
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4
	;zapnout tlacitka
	cli
	in	R16,timsk
	ori	R16,0b10000000
	out	timsk,R16
	sei
	ret
	;write the INFO on the LCD
LCDwriteStatus:	;blokovat tlacitka
	cli
	in	R16,timsk
	andi	R16,0b01111111
	out	timsk,R16
	sei
	;
	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16
	;
	ldi	R16,0b11000000
	call	LCDSetPos
	;vypis cisla radku / cisla listu / poctu listu
	lds	R16,PatternPos
	ldi	R18,64
	sub	R18,R16
	call	bin2BCD8
	mov	R16,R17
	call	LCDwrChar
	mov	R16,R18
	call	LCDwrChar
	ldi	R16,'/'
	call	LCDwrChar
	lds	R18,PatternPTR
	inc	R18
	call	bin2BCD8
	call	LCDwrChar
	mov	R16,R17
	call	LCDwrChar
	mov	R16,R18
	call	LCDwrChar
	ldi	R16,'/'
	call	LCDwrChar	
	lds	R18,PatternN
	dec	R18
	call	bin2BCD8
	call	LCDwrChar
	mov	R16,R17
	call	LCDwrChar
	mov	R16,R18
	call	LCDwrChar
	;
	ldi	R16,0b11001101
	call	LCDsetPos
	;
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4
	;zapnout tlacitka
	cli
	in	R16,timsk
	ori	R16,0b10000000
	out	timsk,R16
	sei
	ret
;********************************************************************************************
;	rutiny nacitani a ukladani skladeb
;	routines for loading/storing songs to the HDD
;********************************************************************************************
;--------------------------------------------------------------------------------------------------------
;	ulozit skladbu na HDD/CF
;	save song to the HDD
;--------------------------------------------------------------------------------------------------------
SaveSongIDE:	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16
	;
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	;check song len
	lds	R16,SongLen+0
	lds	R17,SongLen+1
	clr	R0
	cp	R16,R0
	cpc	R17,R0
	brne	SkladbaReady
	;neni informace o delce - je treba nahrat skladbu znovu
	ldi	R30,low(TextNNSzCT*2)
	ldi	R31,high(TextNNSzCT*2)
	call	LCDwrStrFLASH
	ldi	R16,200
	call	DelayLongVar
	rjmp	SaveSongIDEok
	;skladba je pripravena a je znama delka
	;song is ready to store
SkladbaReady:	lds	R16,SongN
	cpi	R16,255	;check maximum songs count
	brne	NeMaxNSongs
	;dosazen maximalni pocet skladeb (alokacni tabulka plna)
	ldi	R30,low(TextSMPT*2)
	ldi	R31,high(TextSMPT*2)
	call	LCDwrStrFLASH
	ldi	R16,200
	call	DelayLongVar
	rjmp	SaveSongIDEok
	;v alokacni tabulce je jeste misto
NeMaxNSongs:	;ted se projde cela alokacni tabulka, hleda se misto mezi skladbami, pripadne na koci tabulky
	;here the FAT is scanned for the first gap
	;nejdriv nacist alokacni tabulku
	call	LoadIDE_AT	;load FAT to the RAM
	;najit misto mezi skladbami
	ldi	R30,low(OutputFIFO+2)
	ldi	R31,high(OutputFIFO+2)
	;
	ldi	R21,255	;255 bunek
HledejVolbAT:	ld	R19,Z+	;konec predchoziho
	ld	R18,Z+
	cpi	R21,255
	brne	PrvniB_AT
	;kdyz je prvni vzorek, tak nahrat misto konce predchoziho zacatek datoveho prostoru (2)
	ldi	R18,2
	ldi	R19,0
	;
PrvniB_AT:	ld	R17,Z+	;zacatek soucasneho
	ld	R16,Z+
	;test na prazdnou bunku
	clr	R0
	cp	R16,R0
	cpc	R17,R0
	brne	NeVolnabAT
	;nasla se prazdna bunka (na konci vsech jiz nahranych)
	;misto do konce pameti
	lds	R16,MemSize+0
	lds	R17,MemSize+1
	sub	R16,R18
	sbc	R17,R19
	subi	R16,2
	sbci	R17,0
	brcs	JoSaveMaloMem
	;vejde se tam skladba?
	lds	R14,SongLen+0
	lds	R15,SongLen+1
	sub	R16,R14
	sbc	R17,R15
	brcc	JoVolnabAT1	;vejde se
	;nevejde se
JoSaveMaloMem:	rjmp	SaveMaloMem
	;volna a vejde se
JoVolnabAT1:	sbiw	R30,2	;uzakatel na zacatek bunky
	rjmp	JoVolnabAT
NeVolnabAT:	;je mezi dvojici skladeb misto?
	sub	R16,R18
	sbc	R17,R19
	subi	R16,2
	sbci	R17,0
	brcs	SaveMaloMemSB
	lds	R14,SongLen+0
	lds	R15,SongLen+1
	sub	R16,R14
	sbc	R17,R15
	brcs	SaveMaloMemSB
	;nalezeny skladby, mezi kterymi je dostatek mista
	sbiw	R30,2	;ukazatel na misto budouciho zapisu
	rjmp	SaveMistoMezi
SaveMaloMemSB:	dec	R21
	brne	HledejVolbAT
	
	;nelze ulozit - nedostatek pameti
	;gap found, but there is not enought space to store song
SaveMaloMem:	ldi	R30,low(TextNUNPT*2)
	ldi	R31,high(TextNUNPT*2)
	call	LCDwrStrFLASH
	ldi	R16,200
	call	DelayLongVar
	rjmp	SaveSongIDEok
	;nalezeno misto mezi skladbami, je nutne vytvorit mezeru na udaje
	;gap found, so we have to move all records in FAT by one cell (we need space for neg record)
SaveMistoMezi:	call	CreateZeroCell
	;nalezeno misto, ted uz vlastni vypocet hodnot nove bunky
	;now sotore new song cell (start, end) 
JoVolnabAT:	movw	R28,R18
	movw	R26,R18
	adiw	R28,1
	st	Z+,R29
	st	Z+,R28	;zacatek skladby do bunky
	lds	R16,SongLen+0
	lds	R17,SongLen+1
	add	R26,R16
	adc	R27,R17
	adiw	R26,1
	st	Z+,R27
	st	Z+,R26	;konec skladby do bunky
	;ulozit alokacni tabulku zpet
	call	SaveIDE_AT	;and save it to the HDD again (how complicated :-)
	;nastavit adresu zapisu
	;now we have to store the song data at the right psition
	clr	R30
	lsl	R28	
	rol	R29
	rol	R30
	lsl	R28	
	rol	R29
	rol	R30
	mov	R7,R28
	mov	R8,R29
	mov	R9,R30
	clr	R10	;R7:R8:R9:R10 - LBA
	;some LCD info
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextUDT*2)
	ldi	R31,high(TextUDT*2)
	call	LCDwrStrFLASH	;ukladam data na
	call	LCDwrStrCFHDDa	;HDD/CF
	ldi	R16,0b11000000
	call	LCDsetPos

	;konstanty pro statusbar
	lds	R20,SongLen+0
	lds	R21,SongLen+1
	lsl	R20
	rol	R21
	;
	ldi	R16,12	;pro LCD ukazatel
	mul	R20,R16
	mov	R25,R1
	mul	R21,R16
	add	R25,R0
	mov	R24,R25	;konst. pro LCD ukazatel
	;
	clr	R14
	clr	R15
	inc	R14

	clr	R3
	clr	R4
	clr	R5
	
	;nejdriv nacist 2 sektory do interni RAM
	;so now we have to read 1024B into the internal RAM from external SRAM 
NextBlokIDE:	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4	;RAM OE

	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)

	ldi	R18,4	;1024B (2 sektory)
	ldi	R19,0
	;512B
CistBlokIDE:	out	porta,R5
	cbi	portd,5
	sbi	portd,5
	out	porta,R4
	cbi	portd,6
	sbi	portd,6
	out	porta,R3
	;
	add	R3,R14
	adc	R4,R15
	adc	R5,R15
	;
	in	R16,pinc
	st	Z+,R16
	dec	R19
	brne	CistBlokIDE
	dec	R18
	brne	CistBlokIDE
		
	;ted ulozit 2 sektory
	;now we have to store these two sectors on the HDD  
	sbi	portd,4	;RAM OE
	ldi	R16,0b11111111
	out	ddrc,R16
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	
	ldi	R18,2
PisBlokIDE1:	call	SetLBAadressWR
	ldi	R19,0
	;
PisBlokIDE:	ld	R16,Z+
	ld	R17,Z+
	out	portc,R17
	out	porta,R16
	;
	cbi	portd,7
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	sbi	portd,7
	nop
	nop
	;
	dec	R19
	brne	PisBlokIDE
	;
	add	R7,R14
	adc	R8,R15
	adc	R9,R15
	adc	R10,R15
	;
	dec	R18
	brne	PisBlokIDE1
	;
	call	LCDwrStatusbar
	;
	subi	R20,1
	sbci	R21,0
	brne	NextBlokIDE
	
SaveSongIDEok:	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	;prepocitat skladby
	;count song on the disk and job is done
	call	LoadIDE_AT
	call	CountSongsAT
	sts	SongN,R19
	;
	ldi	R16,0
	sts	SongLen+0,R16
	sts	SongLen+1,R16
	;
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4
	ret
;--------------------------------------------------------------------------------------------------------
;	mazat skladbu nebo formatovat HDD/CF (neboli mazat vse)
;	erase song/all/nothigh/info routine
;--------------------------------------------------------------------------------------------------------
SongIDEeraseF:	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16
	;LCD write menu
NeForB:	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextMvolT*2)
	ldi	R31,high(TextMvolT*2)
	call	LCDwrStrFLASH	;napsat mazat skladbu/vse
	ldi	R16,10
	call	DelayLongVar
DRbAg1:	call	WaitButPressed
	andi	R19,0b00001111
	cpi	R19,0
	breq	DRbAg1
	bst	R19,2
	brtc	NeInfiSB
	;zobrazit info o skladbe
	;button - show song info
	call	LCDwrInfSongCF
	rjmp	NeForB
NeInfiSB:	bst	R19,3
	brtc	NeEraseNo
	;nemazat nic
	;button - do nothing
	rjmp	EraseNo
NeEraseNo:	bst	R19,1
	brtc	NeFormatB
	;formatovat, Ano/Ne?
	;button erase - ask Y/N
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextForUT*2)
	ldi	R31,high(TextForUT*2)
	call	LCDwrStrFLASH	;napis urcite formatovat?	
	ldi	R16,10
	call	DelayLongVar
DRbAg2:	call	WaitButPressed
	andi	R19,0b00000011
	cpi	R19,0
	breq	DRbAg2
	bst	R19,1
	brts	NeForB	;nechce formatovat
	bst	R19,0
	brtc	DRbAg2	;tohle se nestane, ale pro jistotu
	;chce formatovat
	;erase all Y - create new FAT
	call	FormatAlTab	;smazat alokacni tabulku
	rjmp	MFoutCsongs
NeFormatB:	;vybral mazat zvolenou skladbu
	;button - erase song
	lds	R16,BitVars
	bst	R16,6
	brtc	NeRAMplEs
	;cte se z RAM - nelze mazat
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextRRNMT*2)
	ldi	R31,high(TextRRNMT*2)
	call	LCDwrStrFLASH	;napis nelze mazat - cte se z RAM
	ldi	R16,200
	call	DelayLongVar
	rjmp	NeForB	;navrat na zakladni menu
	;cte se z HDD/CF - mozna lze mazat
NeRAMplEs:	lds	R16,SongN
	cpi	R16,0
	brne	EraseCanBeB
	;neni co mazat, takze napsat neni co mazat
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextMSvolT*2)
	ldi	R31,high(TextMSvolT*2)
	call	LCDwrStrFLASH	;napis neni co mazat
	ldi	R16,200
	call	DelayLongVar
	rjmp	NeForB	;navrat na zakladni menu
	;je co mazat, takze otazka mazat: ano/ne?
EraseCanBeB:	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextMSvolT*2)
	ldi	R31,high(TextMSvolT*2)
	call	LCDwrStrFLASH	;napis mazat ano/ne?
	ldi	R16,10
	call	DelayLongVar
DRbAg3:	call	WaitButPressed
	andi	R19,0b00000011
	cpi	R19,0
	breq	DRbAg3
	bst	R19,1
	brtc	MoznaEraseSB
	;nechce mazat
	rjmp	NeForB	;navrat na zakladni menu
MoznaEraseSB:	bst	R16,0
	brts	JoMazatSB
	;tohle se nestane, ale pro jistou
	rjmp	DRbAg3
	;chce mazat - tady uz konecne vlastni mazani
JoMazatSB:	call	LoadIDE_AT	;load FAT
	call	ErSongPTRsong	;erase this song record (FAT cell) 
	call	RemFreeCellAT	;remove emty cell (move cells behind it to the left)
	call	SaveIDE_AT	;store FAT
	;prepocitaji se skladby
MFoutCsongs:	call	CountSongNLCDwr
	ldi	R16,200
	call	DelayLongVar
	;tad jeste neco podelat s ukazatelem skladby - jestli se mazala posledni skladba, vynuluje se
	lds	R16,SongN
	lds	R17,SongPTR
	cp	R16,R17	;kdyz posledni
	brne	EraseNo
	;posledni
	clr	R16
	sts	SongPTR,R16
	;vycistit LCD
EraseNo:	call	LCDclrAll
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4
	ret
	
	;vypise info o skladbe
	;show INFO
LCDwrInfSongCF:	call	LoadIDE_AT	;nacist alok. tabulku
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextSJVT*2)
	ldi	R31,high(TextSJVT*2)
	call	LCDwrStrFLASH	;napis "skladba " 
	clr	R17
	lds	R16,SongPTR
	inc	R16
	call	Bin2BCD16
	call	LCDwrBCD	;napis cislo skladby
	ldi	R30,low(TextSMAT*2)
	ldi	R31,high(TextSMAT*2)
	call	LCDwrStrFLASH	;napis " ma "
	ldi	R30,low(OutputFIFO+4)
	ldi	R31,high(OutputFIFO+4)
	lds	R16,SongPTR
	clr	R17
	lsl	R16
	rol	R17
	lsl	R16
	rol	R17
	add	R30,R16
	adc	R31,R17
	ld	R19,Z+
	ld	R18,Z+	;zacatek
	ld	R17,Z+
	ld	R16,Z+	;konec
	sub	R16,R18
	sbc	R17,R19	;velikost skladby
	lsl	R16
	rol	R17	;basobit 2x na kB
	call	Bin2BCD16
	call	LCDwrBCD	;napis velikost skladby
	ldi	R30,low(TextKBT*2)
	ldi	R31,high(TextKBT*2)
	call	LCDwrStrFLASH	;napis "kB"
	;druhy radek
	ldi	R16,0b11000000
	call	LCDsetPos
	call	LCDwrStrCFHDDa
	ldi	R16,':'
	call	LCDwrChar
	ldi	R16,' '
	call	LCDwrChar
	call	CountSongsAT
	movw	R16,R28
	lsl	R16
	rol	R17
	call	bin2bcd16	;vyuzita pamet
	call	LCDwrBCD
	ldi	R16,'/'
	call	LCDwrChar
	lds	R16,MemSize+0
	lds	R17,MemSize+1
	lsl	R16
	rol	R17
	call	bin2bcd16	;celkova pamet
	call	LCDwrBCD
	ldi	R30,low(TextKBT*2)
	ldi	R31,high(TextKBT*2)
	call	LCDwrStrFLASH	;napis "kB"
	ldi	R16,0
	call	DelayLongVar
	ret
;--------------------------------------------------------------------------------------------------------	
;	nacteni songu pres USART
;	load new song via USART 
;--------------------------------------------------------------------------------------------------------
USARTload:	;pripravit RAM pro zapis
	sbi	portd,4	;RAM OE
	ldi	R16,0b11111111
	out	ddrc,R16
	;info
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextCNDT*2)
	ldi	R31,high(TextCNDT*2)
	call	LCDwrStrFLASH
	ldi	R16,0b11000000
	call	LCDsetPos
	;nastavit adresu RAM na zacatek
	clr	R3
	clr	R4
	clr	R5
	clr	R14
	inc	R14
	clr	R15
	in	R16,udr	;vyprazdnit prijimaci registr (jen pro jistotu)
	in	R16,udr
	in	R16,udr
	in	R16,udr	
	;start timeout CTR
	ldi	R16,low(32000)	;asi 2s timeout
	ldi	R17,high(32000)
	out	ocr1ah,R17
	out	ocr1al,R16
	clr	R0
	out	tcnt1h,R0
	out	tcnt1l,R0
	ldi	R16,0b00010000
	out	timsk,R16
	ldi	R16,0b11111111
	out	tifr,R16
	sei
	clr	R25
	;prijimana data zapisovat a inkrementovat adresu
	;receive and store data 
RxByteD:	cpi	R25,255
	breq	RxComp
	sbis	ucsra,rxc
	rjmp	RxByteD
	in	R16,udr
	out	portc,R16	;data na port
	out	porta,R5
	cbi	portd,5
	sbi	portd,5	
	out	porta,R4
	cbi	portd,6
	sbi	portd,6
	out	porta,R3
	;
	add	R3,R14
	adc	R4,R15
	adc	R5,R15
	ldi	R16,8	;pri prekroceni 524288B dal nezapisovat
	cp	R5,R16
	brcc	PlnaRAM
	;
	cbi	portb,3
	nop
	sbi	portb,3
	;
PlnaRAM:	clr	R0
	out	tcnt1h,R0
	out	tcnt1l,R0
	rjmp	RxByteD
	;stop timeout CTR
RxComp:	cli
	ldi	R16,0b00000000
	out	timsk,R16
	;
	ldi	R16,8
	cp	R5,R16
	brcs	NeDelasi524287
	;kdyz je skladba delsi nez 524287B, oriznout na 524287B
	ldi	R16,255
	ldi	R17,7
	mov	R3,R16
	mov	R4,R16
	mov	R5,R17
NeDelasi524287:	;kdyz nedoslo k prijmu, tak nulovat delku
	ldi	R16,0
	ldi	R17,10
	cp	R3,R17
	cpc	R4,R16
	cpc	R5,R16
	brcc	JoPrijemC
	;nedoslo k prijmu
	clr	R20
	clr	R21
	rjmp	NePrijemC 
	;delka skladby ve ctvericich sektoru
JoPrijemC:	lsr	R5
	ror	R4
	lsr	R5
	ror	R4
	lsr	R5
	ror	R4
	ldi	R20,1
	ldi	R21,0
	add	R20,R4
	adc	R21,R5
	;ulozit delku do zasobniku
NePrijemC:	sts	SongLen+0,R20
	sts	SongLen+1,R21
	rjmp	LoadCom	;pokracovat v nacteni songu (pokracuje nize)

;--------------------------------------------------------------------------------------------------------
;	nacteni skladby z CF/HDD
;	loat song from the HDD
;--------------------------------------------------------------------------------------------------------
LoadStartAd:	;vymazat delku skladby (aby se nedalo ukladat v pripade nacetni chybnych dat nebo skladby z CF/HDD)
	clr	R16
	sts	SongLen+1,R16
	sts	SongLen+0,R16
	;
	sbi	portd,4
	ldi	R16,0b11111111
	out	ddrc,R16
	;info
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextNDT*2)
	ldi	R31,high(TextNDT*2)
	call	LCDwrStrFLASH	;nacitam data z
	call	LCDwrStrCFHDDa
	ldi	R16,0b11000000
	call	LCDsetPos

	clr	R3
	clr	R4
	clr	R5
	
	;nejdriv nacist alokacni tabulku
	call	LoadIDE_AT	;load FAT
	;
	ldi	R30,low(OutputFIFO+4)
	ldi	R31,high(OutputFIFO+4)	;prvni bunka	
	lds	R16,SongPTR	;cislo skladby
	clr	R17
	lsl	R16	;nasobit 4x (bunka ma 4B)
	rol	R17
	lsl	R16
	rol	R17
	add	R30,R16	;pricist k ukazateli
	adc	R31,R17
	;nastavit pocatecni LBA adresu
	;set LBA to the start of song 
	ld	R17,Z+
	ld	R16,Z+
	clr	R18
	lsl	R16
	rol	R17
	rol	R18
	lsl	R16
	rol	R17
	rol	R18
	mov	R7,R16	;zacatek souboru (pocatecni sektor)
	mov	R8,R17
	mov	R9,R18
	clr	R10
	;ted spocitat delku v sektorech/2
	;how many sectors/2
	sbiw	R30,2
	ld	R29,Z+
	ld	R28,Z+
	ld	R27,Z+
	ld	R26,Z+
	sub	R26,R28
	sbc	R27,R29
	lsl	R26
	rol	R27
	adiw	R26,1
	movw	R28,R26
	;konstanta pro LCD statusbar
	ldi	R16,12
	mul	R28,R16
	mov	R25,R1
	mul	R29,R16
	add	R25,R0
	mov	R24,R25	;konst. pro LCD ukazatel
	;
	ldi	R16,2
	mov	R13,R16
	clr	R14
	inc	R14	;1
	clr	R15	;0

	;cteni bloku 1024B (2 sektory) z HDD/CF
	;read 1024B
KopirujSong:	call	SetLBAadressRD2

	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R27,2	;2 sektory najednou do zasobniku
	
CistBlok1:	call	WaitDataReady
	call	SetDriveDataReg
	
	ldi	R16,0b00000000
	out	ddra,R16
	out	ddrc,R16

	ldi	R18,0	;512B sektor
CistBlok:	nop
	nop
	cbi	portd,3
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	in	R16,pina
	in	R17,pinc
	sbi	portd,3
	st	Z+,R16
	st	Z+,R17
	nop
	nop
	nop
	;
	dec	R18
	brne	CistBlok
	;
	ldi	R16,0b11111111
	out	ddra,R16
	out	ddrc,R16
	;
	dec	R27
	brne	CistBlok1
	
	;zapis bloku do RAM
	;store it to the external RAM
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R19,0	;1024B blok
	ldi	R18,4
	
PisBlok:	ld	R16,Z+
	out	portc,R16	;data na port
	out	porta,R5
	cbi	portd,5
	sbi	portd,5	
	out	porta,R4
	cbi	portd,6
	sbi	portd,6
	out	porta,R3
	;
	add	R3,R14
	adc	R4,R15
	adc	R5,R15
	;
	cbi	portb,3
	sbi	portb,3
	;
	dec	R19
	brne	PisBlok
	dec	R18
	brne	PisBlok
	;dalsi dvojice sektoru
	add	R7,R13	;2
	adc	R8,R15	;0
	adc	R9,R15	;0
	adc	R10,R15	;0
	;statusbar
	call	LCDwrStatusBar
	;
	subi	R28,1
	sbci	R29,0
	breq	NeKopirujSong
	rjmp	KopirujSong
NeKopirujSong:
LoadCom:	;spocitat skladby
	ldi	R16,0b11111111
	out	ddrc,R16
	;
	call	LCDClrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	;
	ldi	R16,0b00000000
	out	ddrc,R16
	cbi	portd,4	;RAM selsect
;--------------------------------------------------------------------------------------------------------	
;	zjisti o jaky typ MODu jde a nacteni adres, kde zacinaji listy a poradi listu
;	now identify song type
	ldi	R17,low(1084)
	ldi	R18,high(1084)
	sts	PatternStart,R17
	sts	PatternStart+1,R18
	ldi	R17,low(950)
	ldi	R18,high(950)
	sts	PatList,R17
	sts	PatList+1,R18	;tohle je definice pro 31-ti nastrojovou verzi
	;ted se testuje, jestli se na adresach 1080 - 1083 nachazi 1 z retezcu identifikujici 31 kanalovou verzi
	;nejdriv se nacte do zasobniku testovany retezec
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R16,low(1080)
	ldi	R17,high(1080)
	movw	R0,R16
	clr	R2
	ldi	R20,4	;4 znaky
LDtrackType:	call	LoadData
	st	Z+,R16
	inc	R0	;dalsi byte
	dec	R20
	brne	LDtrackType
	;ted porovnavat s moznymi retezci pro 31 nastroju
	ldi	R30,low(FileType31sT*2)	;prvni testovany retezec
	ldi	R31,high(FileType31sT*2)
	;
ZatimSedi3:	ldi	R28,low(OutputFIFO)
	ldi	R29,high(OutputFIFO)
	set		;zatim se rovnaji
	ldi	R20,4	;4 znaky
ZatimSedi2:	ld	R16,Y+	;soubor
	lpm	R17,Z+	;testovany znak znak
	cp	R16,R17
	breq	ZatimSedi1
	clt		;uz se nerovnaji
ZatimSedi1:	dec	R20
	brne	ZatimSedi2
	brts	Jo31nastroju	;nalezen odpovidajici retezec
	lpm	R16,Z
	cpi	R16,0xFF	;test na posledni retezec
	brne	ZatimSedi3
	;dotestovao a zadny nesedel - jde o 15-ti nastrojovou verzi
	ldi	R17,low(600)
	ldi	R18,high(600)
	sts	PatternStart,R17
	sts	PatternStart+1,R18
	ldi	R17,low(470)
	ldi	R18,high(470)
	sts	PatList,R17
	sts	PatList+1,R18	;tohle je definice pro 15-ti nastrojovou verzi
Jo31nastroju:	ret
	;retezce pro 31 nastrojove MODy
FileType31sT:	.db	"M.K."
	.db	"M!K!"
	.db	"FLT4"
	.db	0xFF	;konec moznych retezcu
	;pricitat k statusbaru
LCDwrStatusbar:	dec	R25
	brne	NeLCDldInc
	mov	R25,R24
	ldi	R16,7
	call	LCDwrChar
NeLCDldInc:	ret

;*******************************************************************************************
;	rutiny pro nacitani hlavicky MODu
;	this loads the samples addresses to the buffer
;*******************************************************************************************
	;nastaveni listu a radku	
SetPatternPTR:	ldi	R16,1
	sts	PatternPos,R16
	ldi	R16,255
	sts	PatternPTR,R16
	ret
LoadSmplList:	;nejdriv adresa 1. vzorku do R5:R3
	lds	R4,HighestPatN
	inc	R4
	clr	R5
	clr	R3
	lsl	R4
	rol	R5
	lsl	R4
	rol	R5
	lds	R16,PatternStart
	lds	R17,PatternStart+1
	clr	R18
	add	R3,R16
	adc	R4,R17
	adc	R5,R18	;this is first instrument headder address
	;ted uz zapis zacatku, koncu a smycek
	;now we have to compute start, end, loop position and volume and store it to the sample buffer 
	ldi	R16,42
	mov	R0,R16
	clr	R1
	clr	R2
	ldi	R30,low(SmplsAdress)
	ldi	R31,high(SmplsAdress)
	ldi	R25,0
DalsiSmplL:	;stop adresa
	;stop address
	call	LoadData
	mov	R7,R16
	ldi	R16,1
	call	AddR16Adr
	call	LoadData
	mov	R6,R16
	clr	R8
	lsl	R6
	rol	R7
	rol	R8
	add	R6,R3
	adc	R7,R4
	adc	R8,R5
	push	R6
	push	R7
	push	R8
	;hlasitost
	;volume
	ldi	R16,2
	call	AddR16Adr
	call	LoadData
	lsl	R16
	lsl	R16
	subi	R16,1
	mov	R12,R16
	;zacatek smycky
	;loop start
	ldi	R16,1
	call	AddR16Adr
	call	LoadData
	mov	R10,R16
	ldi	R16,1
	call	AddR16Adr
	call	LoadData
	mov	R9,R16	
	clr	R11
	lsl	R9
	rol	R10
	rol	R11
	add	R9,R3
	adc	R10,R4
	adc	R11,R5
	;delka smycky
	;loop len
	ldi	R16,1
	call	AddR16Adr
	call	LoadData
	mov	R14,R16
	ldi	R16,1
	call	AddR16Adr
	call	LoadData
	mov	R13,R16
	clr	R16
	ldi	R17,2
	cp	R13,R17
	cpc	R14,R16
	brcs	NeLoop1
	;smycka aktivni
	clr	R15
	lsl	R13
	rol	R14
	rol	R15
	add	R13,R9
	adc	R14,R10
	adc	R15,R11
	mov	R6,R13
	mov	R7,R14
	mov	R8,R15
	rjmp	Loop1
	;skutecna stop adresa
NeLoop1:	ldi	R16,0b10000000
	mov	R11,R16
Loop1:	;odecist 1 od konce dat
	;now we have to make real end and start correction - some idiotic trackers writes bad addresses of samples  
	ldi	R17,1
	clr	R17
	sub	R6,R16
	sbc	R7,R17
	sbc	R8,R17
	;nejaka zkusma korekce pozic vzorku
	ldi	R16,2
	ldi	R17,0
	add	R3,R16
	adc	R4,R17
	adc	R5,R17
	ldi	R16,1
	sub	R6,R16
	sbc	R7,R17
	sbc	R8,R17
	;uz zapis
	;now store all the values to the buffer
	st	Z+,R12	;volume
	st	Z+,R3
	st	Z+,R4
	st	Z+,R5	;start
	st	Z+,R6
	st	Z+,R7
	st	Z+,R8	;stop
	st	Z+,R9
	st	Z+,R10
	st	Z+,R11	;repeat
	;ted adresa dalsiho vzorku do R5:R3 a dalsi vzorek v pameti
	pop	R5
	pop	R4
	pop	R3
;	ldi	R16,1
;	ldi	R17,0
;	sub	R3,R16	;korekce, kvuli nesmyslnemu umisteni samplu v souboru 
;	sbc	R4,R17
;	sbc	R5,R17
ZadnejSMPL1:	ldi	R16,23
	ldi	R17,0
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	
	inc	R25
	cpi	R25,31
	breq	NeDalsiSMPLl
	rjmp	DalsiSMPLl
NeDalsiSMPLl:	ret
	;pricist R16 k adrese dane R0:R2
AddR16Adr:	clr	R17
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	ret
;--------------------------------------------------------------------------------------------------------	
;	nacte poradi listu do zasobniku, zjisti cilo nejvyssiho
;	this routine load the pattern list into buffer
;--------------------------------------------------------------------------------------------------------
LoadPatternList:	ldi	R30,low(PatternList)
	ldi	R31,high(PatternList)
	lds	R0,PatList
	lds	R1,PatList+1
	clr	R2
	call	LoadData
	inc	R16
	sts	PatternN,R16
	ldi	R16,2
	ldi	R17,0
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	ldi	R20,127	;pocet listu
	clr	R21	;nejvyssi cislo listu
PatternLoad1:	call	LoadData
	st	Z+,R16
	cp	R16,R21
	brlo	PatLower
	mov	R21,R16
PatLower:	ldi	R16,1
	add	R0,R16
	ldi	R16,0
	adc	R1,R16
	adc	R2,R16
	dec	R20
	cpi	R20,0
	brne	PatternLoad1
	;zapis cislo nejvyssiho listu
	sts	HighestPatN,R21
	ret
	;nacte byte z adresy R0:R2 do R16
LoadData:	out	porta,R2
	cbi	portd,5
	sbi	portd,5
	out	porta,R1
	cbi	portd,6
	sbi	portd,6
	out	porta,R0
	nop
	nop
	nop
	in	R16,pinc
	ret
	
;*******************************************************************************************	
;	tady se ctou noty a nastavuji vsechny konstanty pro efekty a prehravani
;	this routine is pattern line reading and decoding the notes
;	also here are all effects initialized 
;*******************************************************************************************
	;nejdriv cislo radku v listu, kdyz 0, tak dalsi list not
	;test for the new pattern
Takt:	lds	R16,PatternPos
	dec	R16
	sts	PatternPos,R16
	breq	NeDekodujList
	rjmp	DekodujList
NeDekodujList:	ldi	R16,64	;reset pocitadla radku
	sts	PatternPos,R16
	;nacist dalsi list not podle pole poradi listu
	;this was last line from pattern, so load next (if there is some more)
	ldi	R28,low(PatternList)
	ldi	R29,high(PatternList)
	lds	R0,PatternPTR
	lds	R16,BitVars
	bst	R16,3
	brts	IncListBut	;nehrat dalsi list, pokud je list nastaven tlacitky
	inc	R0
IncListBut:	andi	R16,0b11110111
	sts	BitVars,R16
	clr	R1
	add	R28,R0
	adc	R29,R1
	sts	PatternPTR,R0
	lds	R1,PatternN
	dec	R1
	cp	R0,R1
	brne	HrajDal	;pokud je posledni list, tak pokyn k prehravani dalsiho songu
	;
	lds	R16,BitVars
	ori	R16,0b00000001	;prehrat dalsi song (nebo znovu z RAM)
	sts	BitVars,R16
	bst	R16,6
	brts	HrajDal	;kdyz se hraje z RAM, tak ne dalsi song
	lds	R16,SongN
	cpi	R16,0	;pamet prazdna?
	brne	DalsiSNnN
NeDalsiSNnN:	lds	R16,BitVars
	ori	R16,0b00000001	;hrat z RAM kdyz je pamet prazdna
	sts	BitVars,R16
	rjmp	HrajDal
DalsiSNnN:	lds	R16,IDEvars
	bst	R16,7
	brtc	NeDalsiSNnN	;kdyz neni HDD/CF
	lds	R16,SongPTR
	inc	R16
	lds	R17,SongN
	cp	R16,R17
	sts	SongPTR,R16
	brne	HrajDal
	clr	R16
	sts	SongPTR,R16
HrajDal:	;vypocitat a nastavit adresu prvniho radku v dalsim liste
	;now we have to read one note
	clr	R0
	ld	R1,Y+
	clr	R2
	lsl	R1
	rol	R2
	lsl	R1
	rol	R2
	lds	R16,PatternStart
	lds	R17,PatternStart+1
	clr	R18
	add	R0,R16
	adc	R1,R17
	adc	R2,R18
	sts	LinePTR,R0
	sts	LinePTR+1,R1
	sts	LinePTR+2,R2	;1.ch note offset i SRAM
	;tady uz nastaveni nekterych promennych, pro vlastni cteni radku
DekodujList:	ldi	R28,low(PlayDataS)
	ldi	R29,high(PlayDataS)	;play buffer (this is used for information about position, volume, period and MANY other things)
	ldi	R16,0
	;tady zacina smycka pro cteni not pro kazdy ze 4 kanalu
	clr	R20	;pro 4 kanaly
	;nejprve precteni 4 bytu jedne "noty"
	;load 4bytes of this note
DalsiKanal:	lds	R0,LinePTR
	lds	R1,LinePTR+1
	lds	R2,LinePTR+2
	call	LoadData
	mov	R24,R16
	andi	R24,0b11110000	;high cislo vzorku
	andi	R16,0b00000011
	mov	R11,R16	;high period
	ldi	R16,1
	ldi	R17,0
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	call	LoadData
	mov	R10,R16	;low period
	ldi	R16,1
	ldi	R17,0
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	call	LoadData
	mov	R22,R16	;this is effect parameter I think
	andi	R22,0b00001111
	swap	R16
	andi	R16,0b00001111
	or	R24,R16	;instrument nuber
	ldi	R16,1
	ldi	R17,0
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	call	LoadData
	mov	R21,R16	;effect byte
	ldi	R16,1
	ldi	R17,0
	add	R0,R16
	adc	R1,R17
	adc	R2,R17
	sts	LinePTR,R0
	sts	LinePTR+1,R1
	sts	LinePTR+2,R2	;next note
	;zpracovani hlavnich dat (novy vzorek, ton, hlasitost)
	;now we have to do something with this garbage in the registers ;-)
	;here are some exceptions for the playing next note, because of non-comaptibility 
	;of many trackers, so here are many conditions (some times I don't what is it, because it's long time ...)
	clt
	clr	R23
	cpi	R24,0
	brne	NeZadnejVzorek
	clr	R16
	cp	R10,R16
	cpc	R11,R16
	breq	NovyTon
	set
	dec	R24
	rjmp	NeNovyTon
NovyTon:	adiw	R28,19
	rjmp	ZadnejVzorek
NeZadnejVzorek:	dec	R24
	;nastaveni tonu samplu
	;load instrument period
	clr	R16
	cp	R10,R16
	cpc	R11,R16
	brne	NeNovyTon
	;if period missing (exception)
	adiw	R28,6	;kdyz neni ton ale jen cislo samplu
	ser	R23
	ld	R16,Y
	cp	R16,R24
	breq	NovyTonOk
	st	Y+,R24
	clr	R0	;kdyz je jen cislo samplu bez tonu, ale je jiny nez minule
	rjmp	JenNsmpluJiny	;hlasitost na 0
	;now we have to identify period ID - needed for arpeggio
NeNovyTon:	ldi	R16,38	;nacteni cisla tonu pro Arpeggio
	mul	R11,R16
	mov	R2,R0
	mul	R10,R16
	add	R1,R2
	ldi	R16,8
	sub	R1,R16
	clr	R0
	ldi	R30,low(freqT*2)
	ldi	R31,high(freqT*2)
	add	R30,R1
	adc	R31,R0
	lpm	R0,Z+
	st	Y+,R0	;save ID
	;vypocet presne konstanty frekvence z nesmyslnych cisel v souboru :-)
	;now we have to convert something called pariod to the resampling constant
	ldi	R16,low(41088)	;41088/T = freknece prehravani
	ldi	R17,high(41088)
	movw	R18,R10	;nactena perioda T
	call	div16u	;ano deleni a pekne dlouhe ...
	;
	st	Y+,R16	
	st	Y+,R17	;store it, but this is only reserve
	;
	clr	R1
	st	Y+,R1	;this is fraction
	;now if the tone-portamento is active, we cannot change period immediately, but slowly
	cpi	R22,3
	brne	TonePortLF	;pri tone-portamneto nemenit hlavni frekvenci
	cpi	R21,100	;kdyz je portamento speed moc velka, tak skokem
	brcs	NeTonePortLF
	clr	R22	;a neprovadet portamento
	clr	R21
	rjmp	TonePortLF
NeTonePortLF:	adiw	R28,2
	rjmp	NovyTonOk
	;looks like there is no tone-portamento
TonePortLF:	st	Y+,R16
	st	Y+,R17	;store main constant
	;hlasitost, start, stop a repeat pozice
	;volume, start, stop, repeat to the buffer
NovyTonOk:	cpi	R24,255
	brne	MinulySMPLnewf
	ld	R24,Y
MinulySMPLnewf:	st	Y+,R24
	ldi	R16,10
	mul	R16,R24
	ldi	R30,low(SmplsAdress)
	ldi	R31,high(SmplsAdress)
	add	R30,R0
	adc	R31,R1
	ld	R0,Z+
JenNsmpluJiny:	clr	R2
	st	Y+,R2	;pomocna hlasitost
	brtc	NeMenitVol
	ld	R0,Y
NeMenitVol:	st	Y+,R0	;hlavni hlasitost
	;for some effects we can not update buffer values 
	cpi	R22,6
	breq	TportNeChSMPLP	;kdyz efekt 6. tak neobnovovat adresu vzorku
	cpi	R22,5
	breq	TportNeChSMPLP	;kdyz efekt 5. tak neobnovovat adresu vzorku
	cpi	R22,3
	breq	TportNeChSMPLP	;kdyz efekt 3. tak neobnovovat adresu vzorku
	cpi	R23,0
	breq	NastPosSMPL	;kdyz je jen cilso samplu bez tonu
TportNeChSMPLP:	clr	R23	;tak nemenit adresy samplu
	adiw	R28,10
	rjmp	ZadnejVzorek
NastPosSMPL:	clr	R0
	st	Y+,R0
	ldi	R16,9	;9 bytu
CopySdata:	ld	R0,Z+
	st	Y+,R0
	dec	R16
	brne	CopySdata
	;neco pro soft start
	;soft start init
	ldi	R30,low(SMPLsoftStart)
	ldi	R31,high(SMPLsoftStart)
	mov	R16,R20
	ldi	R17,3
	mul	R16,R17
	add	R30,R0
	adc	R31,R1
	ldi	R16,0b11000000	;soft start - hlasitost pomalu dolu
	st	Z,R16
	
;--------------------------------------------------------------------------------------------------------
;	Now fun beggins! Effects initialization :-(
;--------------------------------------------------------------------------------------------------------
ZadnejVzorek:	;tady se uz detekuji a nastavuji jednotlive efekty
	;C - set volume
	cpi	R22,0xC
	brne	NeSetVolume
	cpi	R21,0
	breq	SetTicho
	cpi	R21,40
	brlo	NedelalToAmater
	;nekteri autori pisou az 64, misto 40 - nevedi, ze je to v hexa :-)
	;some assholes don't know what is it hexa, so they write '64', but it's in hexa, so max vol is only 0x40  
	ldi	R21,0x40 
NedelalToAmater:	lsl	R21
	lsl	R21
	subi	R21,1
SetTicho:	sbiw	R28,12
	clr	R0
	st	Y+,R0
	st	Y,R21
	adiw	R28,11
	;5 - volume slide + tone-portamento continue
NeSetVolume:	cpi	R22,5
	breq	VolSlideO
	;6 - volume slide + vibrato continue
	cpi	R22,6
	breq	VolSlideO
	;A - volume slide
	cpi	R22,0xA
	brne	NeVolumeSlide
VolSlideO:	cpi	R21,0
	breq	MoznaSetSpeed
	ldi	R30,low(VolSlideStack)
	ldi	R31,high(VolSlideStack)
	mov	R16,R20
	lsl	R16
	clr	R17
	add	R30,R16
	adc	R31,R17
	;
	ldi	R16,130
	mov	R17,R21
	andi	R17,0b00001111
	cpi	R17,0
	breq	VolSlideUp
	;pro slide down
	mul	R17,R16
	rjmp	VolSlideOk
VolSlideUp:	mov	R17,R21
	swap	R17
	andi	R17,0b00001111
	cpi	R17,0
	breq	VolSlideOk
	mul	R17,R16
	ldi	R16,0b10000000
	or	R1,R16
VolSlideOk:	st	Z+,R0
	st	Z+,R1
	rjmp	MoznaSetSpeed
	;vypnout volume slide
NeVolumeSlide:	ldi	R30,low(VolSlideStack)
	ldi	R31,high(VolSlideStack)
	mov	R16,R20
	lsl	R16
	clr	R17
	add	R30,R16
	adc	R31,R17
	ldi	R16,0
	st	Z+,R16
	st	Z+,R16
	;F - set speed/tempo
MoznaSetSpeed:	cpi	R22,0xF
	brne	NeSetSpeed
	cpi	R21,33
	mov	R16,R21
	brcc	SetTempo
	ldi	R30,low(SpeedTab*2)
	ldi	R31,high(SpeedTab*2)
	clr	R0
	sts	SongSpeed,R16
	lsl	R16
	add	R30,R16
	adc	R31,R0
	lpm	R16,Z+
	lpm	R17,Z+
	sts	PatIncSpeed+2,R16
	sts	PatIncSpeed+3,R17
	lds	R21,PatIncSpeed+4
SetTempo:	sts	PatIncSpeed+4,R21
	lds	R16,PatIncSpeed+2
	lds	R17,PatIncSpeed+3
	subi	R21,33
	ldi	R30,low(TempoKorekceT*2)
	ldi	R31,high(TempoKorekceT*2)
	clr	R0
	add	R30,R21
	adc	R31,R0
	lpm	R18,Z
	mul	R17,R18
	movw	R4,R0
	mul	R16,R18
	clr	R2
	add	R1,R4
	adc	R2,R5
	;delit 64
	lsl	R0
	rol	R1
	rol	R2
	lsl	R0
	rol	R1
	rol	R2
	sts	PatIncSpeed+2,R1
	sts	PatIncSpeed+3,R2
	;D - break pattern, only pattern break
NeSetSpeed:	cpi	R22,0xD
	brne	NeBreak
	ldi	R16,1
	sts	PatternPos,R16
	;9 - set instrument start offset
NeBreak:	cpi	R22,9
	brne	NeSetPos
	clr	R0
	clr	R16
	sbiw	R28,8	;navrat na pozici vzorku
	ld	R1,Y+
	ld	R2,Y+
	clr	R0
	add	R1,R21
	adc	R2,R16
	sbiw	R28,4	;navrat na pozici vzorku
	st	Y+,R0
	st	Y+,R0
	st	Y+,R1
	st	Y+,R2
	adiw	R28,6
	;1 - portamento up (plynule zvyseni tonu)
NeSetPos:	cpi	R22,1
	brne	NePortamentoUp
	call	LoadPortStackAd
	call	PortSpeedKor
	;	
	ldi	R16,0b00100000
	or	R1,R16
	st	X+,R0
	st	X+,R1
	rjmp	NePortUpOff
	;
NePortamentoUp:	call	LoadPortStackAd
	adiw	R26,1
	ld	R16,X
	bst	R16,6
	brts	NePortUpOff
	bst	R16,7
	brts	NePortUpOff
	clr	R16
	st	X+,R16
	;2 - portamento down (plynule snizeni tonu)
NePortUpOff:	cpi	R22,2
	brne	NePortDown
	call	LoadPortStackAd
	call	PortSpeedKor
	;	
	ldi	R16,0b10100000
	or	R1,R16
	st	X+,R0
	st	X+,R1
	rjmp	NePortDownOff
	;
NePortDown:	call	LoadPortStackAd
	adiw	R26,1
	ld	R16,X
	bst	R16,6
	brts	NePortDownOff
	bst	R16,7
	brtc	NePortDownOff
	clr	R16
	st	X+,R16
	;3 - tone-portamento plynuly prechod na jinou notu
NePortDownOff:	cpi	R22,3
	brne	NeTonePort
	call	LoadPortStackAd
	sbiw	R28,18
	ld	R16,Y+
	ld	R17,Y+
	adiw	R28,1
	ld	R18,Y+
	ld	R19,Y+
	adiw	R28,13
	cp	R16,R18
	cpc	R17,R19
	breq	NeTonePortNotEq
	rjmp	TonePortNotEq
	;nemenit
NeTonePortNotEq:	ldi	R19,0b00000000
	rjmp	TonePortCom
TonePortNotEq:	brcs	TonePortD
	;zvysovani
	ldi	R19,0b01100000
	rjmp	TonePortCom
	;snizovani
TonePortD:	ldi	R19,0b11100000
TonePortCom:	call	PortSpeedKor
	cpi	R21,0
	brne	NeTonePortNK	;kdyz uz ma jen pokracovat R21=0
	ld	R0,X+
	ld	R1,X+
	sbiw	R26,2
NeTonePortNK:	or	R1,R19
	st	X+,R0
	st	X+,R1
	rjmp	NeTonePortOff
	;vypnout tone portamento
NeTonePort:	cpi	R22,5	;pro efekt 5
	breq	NeTonePortOff
	cpi	R22,6	;a pro efekt 6 nevypinat port
	breq	NeTonePortOff
	call	LoadPortStackAd
	adiw	R26,1
	ld	R16,X
	bst	R16,6
	brtc	NeTonePortOff
	andi	R16,0b00011111
	st	X+,R16
	;0 - Arpeggio (skokova zmena tonu mezi 3 tony) / nic, pac 0 ...
NeTonePortOff:	cpi	R22,0
	breq	JoArpeggio
	rjmp	NeArpeggio
JoArpeggio:	cpi	R21,0
	breq	NeArpeggio
	;cislo kanalu
	ldi	R26,low(ArpeggioStack)
	ldi	R27,high(ArpeggioStack)
	mov	R16,R20
	ldi	R17,7
	mul	R16,R17
	add	R26,R0
	adc	R27,R1
	;vypocitat frekvence
	ldi	R30,low(FreqNewT*2)
	ldi	R31,high(FreqNewT*2)
	sbiw	R28,19
	ld	R16,Y	;cislo zakladni frekvence
	adiw	R28,19
	lsl	R16
	clr	R17
	add	R30,R16
	adc	R31,R17
	;ktery ton hrat nejdriv
	ld	R16,X
	cpi	R16,0
	brne	UzJeAktivArp
	ldi	R16,1
UzJeAktivArp:	st	X+,R16
	;nacist konst. pro zakladni frekvenci
	lpm	R18,Z+
	lpm	R19,Z+
	ldi	R16,low(41088)
	ldi	R17,high(41088)
	call	div16u	
	st	X+,R16
	st	X+,R17	;zakladni zapsana
	sbiw	R30,2
	movw	R24,R30
	mov	R16,R21
	andi	R16,0b00001111	;vybrat prvni frekvenci
	lsl	R16
	clr	R17
	sub	R30,R16
	sbc	R31,R17
	lpm	R18,Z+
	lpm	R19,Z+	;nactena frekvence 1
	ldi	R16,low(41088)
	ldi	R17,high(41088)
	call	div16u
	st	X+,R16
	st	X+,R17
	movw	R30,R24
	mov	R16,R21
	swap	R16
	andi	R16,0b00001111	;vybrat druhou frekvenci
	lsl	R16
	clr	R17
	sub	R30,R16
	sbc	R31,R17
	lpm	R18,Z+
	lpm	R19,Z+	;nactena frekvence 2
	ldi	R16,low(41088)
	ldi	R17,high(41088)
	call	Div16u
	st	X+,R16
	st	X+,R17
	rjmp	NeDeArp1
	;toto je ukonceni arpeggia - frekvence se nastavi na puvodni (zalozni)
	;on the end of arpeggio effect
NeArpeggio:	;cislo kanalu
	ldi	R26,low(ArpeggioStack)
	ldi	R27,high(ArpeggioStack)
	mov	R16,R20
	ldi	R17,7
	mul	R16,R17
	add	R26,R0
	adc	R27,R1
	ld	R16,X
	cpi	R16,0	;bylo aktivni?
	breq	NeDeArp1
	clr	R16
	st	X,R16	;vypnout arpeggio
	;vypocitat frekvence
	ldi	R30,low(FreqNewT*2)
	ldi	R31,high(FreqNewT*2)
	sbiw	R28,18
	ld	R16,Y+	;cislo zakladni frekvence
	ld	R17,Y+
	clr	R18
	st	Y+,R18
	st	Y+,R16
	st	Y+,R17
	adiw	R28,13
NeDeArp1:	;Ex effects
	cpi	R22,0xE
	breq	EfektyE
	rjmp	NeEfektyE
EfektyE:	mov	R22,R21
	andi	R22,0b11110000
	swap	R22
	andi	R21,0b00001111
	;EA - volume up slowly
	cpi	R22,0xA
	brne	NeFineVolUP
	sbiw	R28,12
	clr	R0
	st	Y+,R0
	ld	R16,Y
	lsl	R21
	lsl	R21
	add	R16,R21	;pricist konstantu
	brcc	NeMaxVolFSU
	ldi	R16,255
NeMaxVolFSU:	st	Y,R16
	adiw	R28,11
	;EB - volume down slowly 
NeFineVolUp:	cpi	R22,0xB
	brne	NeFineVolDown
	sbiw	R28,12
	clr	R0
	st	Y+,R0
	ld	R16,Y
	lsl	R21
	lsl	R21
	sub	R16,R21	;odecist konstantu
	brcc	NeMinVolFSD
	ldi	R16,0
NeMinVolFSD:	st	Y,R16
	adiw	R28,11
NeFineVolDown:	;E1 - portamneto up slowly
	cpi	R22,1
	brne	NeFinePortUp
	sbiw	R28,15
	ld	R16,Y+
	ld	R17,Y+
	sbiw	R28,2
	clr	R18
	add	R16,R21
	adc	R17,R18
	st	Y+,R16
	st	Y+,R17
	adiw	R28,13
NeFinePortUp:	;E2 - portamneto down slowly
	cpi	R22,2
	brne	NeFinePortDown
	sbiw	R28,15
	ld	R16,Y+
	ld	R17,Y+
	sbiw	R28,2
	sub	R16,R21
	sbci	R17,0
	st	Y+,R16
	st	Y+,R17
	adiw	R28,13
NeFinePortDown:	;next note in da channel
NeEfektyE:	inc	R20
	cpi	R20,4
	breq	NeDalsiKanal
	rjmp	DalsiKanal	
NeDalsiKanal:	;uz vypocteny vsechny 4 kanaly
	;all notes done, LCD rewrite
	;zobrazit status na LCD
	call	LCDwriteStatus
	ret
	
	;koeficienty pro portamneto pro ruzne rychlosti
	;portamento consts correction
	;     konstanty	 speed
PortamentoT:	.db	0,1	;0,1
	.db	2,3	;2,3
	.db	4,4	;4,5
	.db	4,4	;6,7
	.db	4,4	;8,9
	.db	4,4	;10,11
	.db	4,4	;12,13
	.db	4,4	;14,15
	
	;tabulka konstant pro prehravaci rychlosti
	;play speed consts
SpeedTab:	.dw	52	;0 - ?????
	.dw	52	;1 - 51.18
	.dw	106	;2 - 25.43
	.dw	162	;3 - 16.83
	.dw	218	;4 - 12.59
	.dw	272	;5 - 10.07
	.dw	323	;6 - 8.47
	.dw	384	;7 - 7.2
	.dw	435	;8 - 6.35
	.dw	492	;9 - 5.63
	.dw	551	;A - 5.03
	.dw	601	;B - 4.61
	.dw	660	;C - 4.2
	.dw	710	;D - 3.91
	.dw	768	;E - 3.61
	.dw	828	;F - 3.35
	.dw	871	;10 - 3.2
	.dw	915	;11 - 3.05
	.dw	976	;12 - 2.85
	.dw	1038	;13 - 2.69
	.dw	1096	;14 - 2.55
	.dw	1160	;15 - 2.4
	.dw	1227	;16 - 2.27
	.dw	1290	;17 - 2.16
	.dw	1328	;18 - 2.1
	.dw	1395	;19 - 2
	.dw	1447	;1A - 1.93
	.dw	1501	;1B - 1.86
	.dw	1547	;1C - 1.8
	.dw	1600	;1D - 1.74
	.dw	1674	;1E - 1.66
	.dw	1700	;1F - 1.64
	.dw	1783	;20 - 1.62

;SpeedTab:	.dw	9	;0 - 
;	.dw	55	;1 - 51.18
;	.dw	110	;2 - 25.43
;	.dw	166	;3 - 16.83
;	.dw	222	;4 - 12.59
;	.dw	277	;5 - 10.07
;	.dw	329	;6 - 8.47
;	.dw	388	;7 - 7.2
;	.dw	439	;8 - 6.35
;	.dw	496	;9 - 5.63
;	.dw	555	;A - 5.03
;	.dw	605	;B - 4.61
;	.dw	664	;C - 4.2
;	.dw	714	;D - 3.91
;	.dw	773	;E - 3.61
;	.dw	833	;F - 3.35

	;prevodni tabulka pro nastaveni tempa
	;tempo correction
TempoKorekceT:	.db	248,241,234,228,221,216,210,205,200,195,191,186,182,178,174,171
	.db	167,164,161,158,155,152,149,146,144,141,139,137,134,132,130,128
	.db	126,124,122,120,119,117,115,114,112,111,109,108,106,105,104,102
	.db	101,100,99,98,96,95,94,93,92,91,90,89,88,87,86,85
	.db	84,84,83,82,81,80,80,79,78,77,77,76,75,74,74,73
	.db	72,72,71,71,70,69,69,68,68,67,67,66,66,65,65,64
	.db	64,63,63,62,62,61,61,60,60,59,59,59,58,58,57,57
	.db	56,56,56,55,55,55,54,54,54,53,53,53,52,52,52,51
	.db	51,51,50,50,50,49,49,49,48,48,48,48,47,47,47,47
	.db	46,46,46,46,45,45,45,45,44,44,44,44,43,43,43,43
	.db	42,42,42,42,42,41,41,41,41,41,40,40,40,40,40,39
	.db	39,39,39,39,38,38,38,38,38,38,37,37,37,37,37,37
	.db	36,36,36,36,36,36,35,35,35,35,35,35,35,34,34,34
	.db	34,34,34,34,33,33,33,33,33,33,33,33,32,32,32,32
	;nacte adresu portamento zasobniku pro R20 kanal
LoadPortStackAd:	ldi	R26,low(PortamentoStack)
	ldi	R27,high(PortamentoStack)
	mov	R18,R20
	lsl	R18
	clr	R16
	add	R26,R18
	adc	R27,R16
	ret
	;korekce na speed pro portamento
PortSpeedKor:	ldi	R30,low(PortamentoT*2)
	ldi	R31,high(PortamentoT*2)
	lds	R16,SongSpeed
	add	R30,R16
	clr	R16
	adc	R31,R16
	lpm	R16,Z
	mul	R21,R16
	ret
	;prekodovani prehravacich rychlosti ((perioda*38)/256)-8
	;this is table for get the period ID
FreqT:	.db	0,2,4,5,7,8,9,11,12,13,14,14,15,16,17,18
	.db	18,19,20,20,21,22,22,23,23,24,24,25,25,26,26,27
	.db	27,28,28,29,29,29,29,30,31,31,31,31,32,32,32,33
	.db	33,33,34,34,34,34,35,35,35,35,36,36,36,36,37,37
	.db	37,37,38,38,38,38,39,39,39,39,40,40,40,40,40,41
	.db	41,41,41,41,41,42,42,42,42,42,43,43,43,43,43,43
	.db	44,44,44,44,44,44,45,45,45,45,45,45,45,46,46,46
	.db	46,46,46,47,47,47,47,47,47,47,47,48,48,48,48,48
	.db	48,48,48,49,49,49,49,49,49,49,49,50,50,50,50,50
	.db	50,50,50,51,51,51,51,51,51,51,51,51,51,52,52,52
	.db	52,52,52,52,52,52,52,53,53,53,53,53,53,53,53,53
	.db	53,54,54,54,54,54,54,54,54,54,54,54,55,55,55,55
	.db	55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56
	.db	56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57
	.db	57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58
	.db	59,59,59,59,59,59,59,59,59,59,59,59,59,59,59,59
	;Tabulka period pro Arpeggio
	;arpeggio periods
FreqNewT:	.dw	57
	.dw	60
	.dw	64
	.dw	67
	.dw	71
	.dw	76
	.dw	80
	.dw	85
	.dw	90
	.dw	95
	.dw	101
	.dw	107
	.dw	113
	.dw	120
	.dw	127
	.dw	135
	.dw	143
	.dw	151
	.dw	160
	.dw	170
	.dw	180
	.dw	190
	.dw	202
	.dw	214
	.dw	226
	.dw	240
	.dw	254
	.dw	269
	.dw	285
	.dw	302
	.dw	320
	.dw	339
	.dw	360
	.dw	381
	.dw	404
	.dw	428
	.dw	453
	.dw	480
	.dw	508
	.dw	538
	.dw	570
	.dw	604
	.dw	640
	.dw	678
	.dw	720
	.dw	762
	.dw	808
	.dw	856
	.dw	907
	.dw	961
	.dw	1017
	.dw	1077
	.dw	1141
	.dw	1209
	.dw	1281
	.dw	1357
	.dw	1440
	.dw	1525
	.dw	1616
	.dw	1712

;*******************************************************************************************
;	Tady jsou vsechny rutiny pro LCD
;	LCD routines (no comment)
;*******************************************************************************************
	;vymaze cele LCD
LCDclrAll:	sbi	portb,1
	ldi	R25,128
LCDclrAll1:	ldi	R16,' '
	call	ByteToLCD
	dec	R25
	brne	LCDclrAll1
	ret
	;nastavi pozici na LCD podle R16
LCDsetPos:	cbi	portb,1
	call	ByteToLCD
	ret
	;posle znak z R16 na LCD
LCDwrChar:	sbi	portb,1
	call	ByteToLCD
	ret
	;vypise retezec z od adresy udane Z, retezec se ukoncuje 0xFF, radek zalamuje 0x00
LCDwrStrFlash:	sbi	portb,1
LCDwrStrFlash1:	lpm	R16,Z+
	call	ByteToLCD
	lpm	R16,Z
	cpi	R16,0
	brne	LCDNeBrLine
	ldi	R16,0b11000000
	call	LCDsetPos
	sbi	portb,1
	adiw	R30,1
LCDNeBrLine:	lpm	R16,Z
	cpi	R16,255
	brne	LCDwrStrFlash1
	ret
	;odesle R25 znaku z RAM od ukazatele Z
LCDwrStrRAM:	sbi	portb,1	;data
LCDwrStrRAM1:	ld	R16,Z+
	call	ByteToLCD
	dec	R25
	brne	LCDwrStrRAM1
	ret 	
	;vypise BCD cislo bez nul na zacatku
LCDwrBCD:	sbi	portb,1
	clt
	mov	R16,R15
	andi	R16,0b00001111
	subi	R16,-48
	cpi	R16,48
	breq	NextNMBR1
	set
	call	ByteToLCD
NextNMBR1:	mov	R16,R14
	andi	R16,0b11110000
	swap	R16
	subi	R16,-48
	brts	NMBR2
	cpi	R16,48
	breq	NextNMBR2
	set
NMBR2:	call	ByteToLCD
NextNMBR2:	mov	R16,R14
	andi	R16,0b00001111
	subi	R16,-48
	brts	NMBR3
	cpi	R16,48
	breq	NextNMBR3
	set
NMBR3:	call	ByteToLCD
NextNMBR3:	mov	R16,R13
	andi	R16,0b11110000
	swap	R16
	subi	R16,-48
	brts	NMBR4
	cpi	R16,48
	breq	NextNMBR4
	set
NMBR4:	call	ByteToLCD
NextNMBR4:	mov	R16,R13
	andi	R16,0b00001111
	subi	R16,-48
	call	ByteToLCD
	ret
	;inicializace LCD jako 2 radkovy
LCDinit:	ldi	R16,0b11111111
	out	ddrc,R16
	call	ByteToLCDst
	ldi	R16,0b00111000
	call	ByteToLCD
	ldi	R16,0b00001100
	call	ByteToLCD
	call	GenerovatZnaky
	ldi	R16,0b10000000
	call	ByteToLCD
	ldi	R16,0b00000000
	out	ddrc,R16
	ret
	;posle byte LCD a generuje zpozdeni neco pres 40us	
ByteToLCD:	out	portc,R16
	sbi	portb,0
	nop
	cbi	portb,0
	ldi	R16,220
ByteToLCDL1:	dec	R16
	nop
	brne	ByteToLCDL1
	ret
	;jen zpozdeni po zapnuti (nektere LCD se nechtely inicializovat)
ByteToLCDst:	ldi	R17,2
	ldi	R16,0	
ByteToLCDst1:	dec	R16
	brne	ByteToLCDst1
	dec	R17
	brne	ByteToLCDst1
	ret
	;vygeneruje znaky pro bargrafy do CGRAM
GenerovatZnaky:	ldi	R19,0b01000000
	ldi	R17,8
	ldi	R18,1
GenZnaky1:	cbi	portb,1
	mov	R16,R19
	inc	R19
	call	ByteToLCD
	sbi	portb,1
	ldi	R16,0b00000000
	cp	R18,R17
	brcs	NeBlack
	ldi	R16,0b00011111
NeBlack:	call	ByteToLCD
	inc	R18
	cpi	R18,9
	brne	DalsiBarCh
	ldi	R18,1
	dec	R17
DalsiBarCh:	cpi	R19,0b10000000
	brne	GenZnaky1
	cbi	portb,1
	ret
	;vypise informace o IDE zarizeni (HDD/CF), nastavi nektere promenne
LCDwrInitInf:	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R16,0b11111111
	out	ddrc,R16
	;je neco pripojeno k IDE?
	ldi	R25,255
	clt
IDEmoznaJe:	ld	R16,Z+
	cpi	R16,0xFF
	breq	IDEzatimNeni
	set
IdeZatimNeni:	dec	R25
	brne	IDEmoznaJe
	;?
	lds	R16,IDEvars
	bld	R16,7	;1 - IDE zarizeni je, 0- neni
	sts	IDEvars,R16
	;
	brts	NaIDEnecoJe
	;na IDE nic neni
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextNoDriveT*2)
	ldi	R31,high(TextNoDriveT*2)
	call	LCDwrStrFLASH	;napis IDE neni
	rjmp	IDEneniSkip
	;IDE zarizeni pripojeno
NaIDEnecoJe:	ldi	R30,low(OutputFIFO+54)
	ldi	R31,high(OutputFIFO+54)
	;vypis "model number"
	;prvni ardek
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R25,20
	call	LCDwrStrRAM
	;druhy radek
	ldi	R16,0b11000000
	call	LCDsetPos
	ldi	R25,20
	call	LCDwrStrRAM
	;cekat 1s
	ldi	R16,80
	call	DelayLongVar
	;vypis typu media, kapacity, CHS
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	lds	R16,OutputFIFO+1
	bst	R16,7	;CF/HDD
	;zapsat HDD/CF
	lds	R16,IDEvars
	bld	R16,0
	sts	IDEvars,R16
	;
	ldi	R30,low(TextCFT*2)
	ldi	R31,high(TextCFT*2)
	brts	IDEjeCF
	ldi	R30,low(TextHDDT*2)
	ldi	R31,high(TextHDDT*2)
IDEjeCF:	call	LCDwrStrFLASH	;napis typ media
	;
	call	SetDriveParam
	;spocitat sektory
	ldi	R30,low(OutputFIFO+6)
	ldi	R31,high(OutputFIFO+6)
	ld	R21,Z+
	ld	R20,Z+
	adiw	R30,4
	ld	R23,Z+
	ld	R22,Z+
	call	mul16u
	ldi	R30,low(OutputFIFO+2)
	ldi	R31,high(OutputFIFO+2)
	ld	R23,Z+
	ld	R22,Z+
	movw	R20,R16
	call	mul16u
	;ulozit pocet ctveric sektoru
	push	R16
	push	R17
	push	R18
	;
	lsr	R18
	ror	R17
	ror	R16
	lsr	R18
	ror	R17
	ror	R16	;delit pocet sektoru 4x
	;
	ldi	R19,255
	ldi	R20,0
	cp	R16,R19
	cpc	R17,R19
	cpc	R18,R20
	brcs	MaxMemSize
	;max. pocet ctveric sektoru je 65535
	ldi	R16,255
	ldi	R17,255
MaxMemSize:	sts	MemSize+1,R17	;velikost pameti ve ctvericich sektoru
	sts	MemSize+0,R16
	;
	pop	R18
	pop	R17
	pop	R16
	;delit na kapacitu v MB
	ldi	R19,low(1959)	;1959
	ldi	R20,high(1959)
	clr	R21
	call	div24u
	;prevod na BCD a vypis
	call	bin2bcd16
	call	LCDwrBCD	;napis velikost
	ldi	R30,low(TextMBT*2)
	ldi	R31,high(TextMBT*2)
	call	LCDwrStrFLASH	;"MB "
	;2. radek
	ldi	R16,0b11000000
	call	LCDSetPos
	ldi	R30,low(TextCHST*2)
	ldi	R31,high(TextCHST*2)
	call	LCDwrStrFLASH	;"CHS "
	lds	R17,OutputFIFO+2
	lds	R16,OutputFIFO+3
	call	bin2bcd16
	call	LCDwrBCD	;pocet cylindru
	ldi	R16,'/'
	call	LCDwrChar
	lds	R17,OutputFIFO+6
	lds	R16,OutputFIFO+7
	call	bin2bcd16
	call	LCDwrBCD	;hlav
	ldi	R16,'/'
	call	LCDwrChar
	lds	R17,OutputFIFO+12
	lds	R16,OutputFIFO+13
	call	bin2bcd16
	call	LCDwrBCD	;sektoru
	ldi	R16,100
	call	DelayLongVar
	;nacteni alokacni tabulky
ZkZnovuAlTab0:	ldi	R16,0b11111111
	out	ddra,R16
	call	LoadIDE_AT
	;posle FAT pres USART
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R18,0
	ldi	R19,4
hhh:	ld	R16,Z+
	out	udr,R16
	ldi	R16,12
	call	DelayShortVar
	dec	R18
	brne	hhh
	dec	R19
	brne	hhh
	;je tam retezec "Tab0" ?
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ld	R16,Z+
	ld	R17,Z+
	ld	R18,Z+
	ld	R19,Z+
	ldi	R20,'T'
	ldi	R21,'a'
	ldi	R22,'b'
	ldi	R23,'0'
	cp	R16,R20
	cpc	R17,R21
	cpc	R18,R22
	cpc	R19,R23
	breq	Tab0Je	;tabulka nalezena
	;tab0 neni
ChAlTabDal:	call	LCDclrAll
	ldi	R16,25
	call	DelayLongVar	;chvili pockej
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextTabNeniT*2)
	ldi	R31,high(TextTabNeniT*2)
	call	LCDwrStrFLASH	;napis neni tabulka, co dal?
	call	LCDwrStrCFHDDa	;HDD/CF
	ldi	R16,5
	call	DelayLongVar	;chvili pockej
ZnovuButRD1:	call	WaitButPressed
	andi	R19,0b00000111
	cpi	R19,0
	breq	ZnovuButRD1
	bst	R19,0
	brts	ZkZnovuAlTab0	;stiskl znovu
	bst	R19,1
	brtc	NeFormatAlTab
	;ano/ne ? (stiskl format)
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextForUT*2)
	ldi	R31,high(TextForUT*2)
	call	LCDwrStrFLASH	;napis skutecne formatovat?
	ldi	R16,5
	call	DelayLongVar	;chvili pockej
ZnovuButRD2:	call	WaitButPressed
	andi	R19,0b00000011
	cpi	R19,0
	breq	ZnovuButRD2
	bst	R19,1
	brts	ChAlTabDal	;stiskl Ne
	bst	R19,0
	brtc	ZnovuButRD2
	;stiskl Ano
	call	FormatAlTab	;formatovat disk (vytvorit prazdnou alokacni tabulku)
	rjmp	ZkZnovuAlTab0	;test jestli se zapsala tabulka
	;
NeFormatAlTab:	bst	R19,2
	brtc	NeBezHDDCF
	;stiskl bez HDD/CF
	lds	R16,IDEvars
	andi	R16,0b01111111	;vynulovat priznak IDE zarizeni (jakoby ho vypnout)
	sts	IDEvars,R16
	ldi	R16,0
	sts	SongN,R16
	rjmp	IDEneniSkip1	;normalne pokracovat, ale bez IDE zarizeni
NeBezHDDCF:	rjmp	ZkZnovuAlTab0	;sem se program nikdy nedostane, ale pro jistotu :-)
	;tabulka nalezena nebo zformatovana, spocitat skladby
Tab0Je:	call	CountSongNLCDwr
	;
IDEneniSkip:	ldi	R16,180
	call	DelayLongVar
	;
IDEneniSkip1:	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ret
	
	;vypise pocet skladeb a kolik zabiraji
CountSongNLCDwr:	call	CountSongsAT
	sts	SongN,R19	;ulozit pocet skladeb
	mov	R16,R19
	clr	R17
	call	bin2bcd16	;retezec s poctem skladeb
	call	LCDclrAll
	ldi	R16,0b10000000
	call	LCDsetPos
	ldi	R30,low(TextNST*2)
	ldi	R31,high(TextNST*2)
	call	LCDwrStrFLASH
	call	LCDwrBCD	;vypis pocet skladeb
	;ted vypis kolik zabiraji
	ldi	R30,low(TextVeT*2)
	ldi	R31,high(TextVeT*2)
	call	LCDwrStrFLASH
	lsl	R28
	rol	R29	;nasobit 2x na velikost v kB
	movw	R16,R28
	call	bin2bcd16
	call	LCDwrBCD	;vypis pocet skladeb
	ldi	R30,low(TextKBT*2)
	ldi	R31,high(TextKBT*2)
	call	LCDwrStrFLASH
	ret
	;napise CF/HDD podle pripojeneho media
LCDwrStrCFHDDa:	lds	R16,IDEvars
	bst	R16,0
	ldi	R30,low(TextCFsT*2)
	ldi	R31,high(TextCFsT*2)
	brts	IDEjeCF1
	ldi	R30,low(TextHDDT*2)
	ldi	R31,high(TextHDDT*2)
IDEjeCF1:	call	LCDwrStrFLASH	;napis typ media
	ret

;*******************************************************************************************
;	tady by mely byt vsechny rutiny pro praci s IDE apod.
;	IDE routines
;*******************************************************************************************
;--------------------------------------------------------------------------------------------
;	nastavi fyzickou geometrii disku, z identifikacniho retezce
;	set physical geometry for HDD - this is absolutely needed for many HDDs!!!!!!!  
;--------------------------------------------------------------------------------------------
SetDriveParam:	ldi	R30,low(OutputFIFO+7)
	ldi	R31,high(OutputFIFO+7)
	ld	R10,Z+	;pocet hlav
	dec	R10	;(pocet hlav)-1
	adiw	R30,5
	ld	R6,Z+	;pocet sektoru na stopu
	;
	rcall	SetLBAadressCom
	rcall	WaitDriveBSY
	;
	ldi	R16,0b11110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ldi	R16,0x91	;cti sektor
	out	portc,R16
	rcall	IOWRpulz
	rcall	WaitDriveBSY
	ret
;--------------------------------------------------------------------------------------------------------
;	nastavi adresu v LBA modu
;	set LBA
;--------------------------------------------------------------------------------------------------------
SetLBAadressCom:	call	WaitDriveBSY
	;D0-D7 jako vystup
	ldi	R16,0b11111111
	out	ddrc,R16
	;pocet sektoru pro pristi zapis/cteni
	ldi	R16,0b10100000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	out	portc,R6	;pocet sektoru zapisovanych najednou
	rcall	IOWRpulz
	;LBA A0-A7 (sektor)
	ldi	R16,0b10110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	out	portc,R7	;LBA A0-A7
	rcall	IOWRpulz
	;LBA A8-A15 (cylindr low)
	ldi	R16,0b11000000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	out	portc,R8	;LBA A8-A15
	rcall	IOWRpulz
	;LBA A16-A23 (cylindr high)
	ldi	R16,0b11010000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	out	portc,R9	;LBA A16-A23
	rcall	IOWRpulz
	;LBA A24-A27 (SDH registr)
	ldi	R16,0b11100000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ldi	R16,0b11000000	;LBA mod
	or	R16,R10	;LBA A24-A27
	out	portc,R16
	rcall	IOWRpulz
	ret
;--------------------------------------------------------------------------------------------------------
;	nastavi LBA adresu a vysle instrukci cteni sektoru
;	set LBA + send read instruction
;--------------------------------------------------------------------------------------------------------
SetLBAadressRD:	ldi	R16,1	;pocet sektoru
	mov	R6,R16
	call	SetLBAadressCom
	call	WaitDriveBSY
	;cti sektor
	ldi	R16,0b11110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ldi	R16,0x20	;cti sektor
	out	portc,R16
	rcall	IOWRpulz
	ret
;--------------------------------------------------------------------------------------------------------
;	nastavi LBA adresu a vysle instrukci cteni 2 sektoru
;	the same for 2 sectors
;--------------------------------------------------------------------------------------------------------
SetLBAadressRD2:	ldi	R16,2	;pocet sektoru
	mov	R6,R16
	call	SetLBAadressCom
	call	WaitDriveBSY
	;cti sektor
	ldi	R16,0b11110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ldi	R16,0x20	;cti sektor
	out	portc,R16
	rcall	IOWRpulz
	ret
;--------------------------------------------------------------------------------------------------------
;	nastavi LBA adresu, vysle instrukci zapisu sektoru, nastavi datovy registr, ceka na busy
;	set LBA, send write instr, set data reg, wait for DRQ
;--------------------------------------------------------------------------------------------------------
SetLBAadressWR:	;nasavit adresu
	ldi	R16,1
	mov	R6,R16	;pocet sektoru
	call	SetLBAadressCom
	call	WaitDriveBSY
	;zapis sektor
	ldi	R16,0b11110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ldi	R16,0x30	;zapis sektoru
	out	portc,R16
	rcall	IOWRpulz
	call	WaitDriveBSY
	call	SetDriveDataReg
	ret
;--------------------------------------------------------------------------------------------------------
;	cekat dokud je HDD busy
;	wait HDD busy
;--------------------------------------------------------------------------------------------------------
WaitDriveBSY:	call	SetDriveStatReg
WaitDriveBSY1:	call	ReadDriveStat
	bst	R16,7
	brts	WaitDriveBSY1
	out	ddrc,R17
	ret
;--------------------------------------------------------------------------------------------------------
;	cekat na data z HDD
;	wait HDD data ready (DRQ)
;--------------------------------------------------------------------------------------------------------
WaitDataReady:	call	SetDriveStatReg
WaitDataReady1:	call	ReadDriveStat
	bst	R16,3
	brtc	WaitDataReady1
	out	ddrc,R17
	ret
;--------------------------------------------------------------------------------------------------------
;	cekat dokud je HDD busy
;	wait untill HDD is ready
;--------------------------------------------------------------------------------------------------------
WaitDriveRDY:	call	SetDriveStatReg
WaitDriveRDY1:	call	ReadDriveStat
	bst	R16,6
	brtc	WaitDriveRDY1
	out	ddrc,R17
	ret
;--------------------------------------------------------------------------------------------------------
;	nacte chybovy registr
;	read error register
;--------------------------------------------------------------------------------------------------------
CheckErrReg:	in	R17,ddrc
	ldi	R16,0b00000000
	out	ddrc,R16
	ldi	R16,0b10010000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	call	ReadDriveStat
	out	ddrc,R17
	ret
;--------------------------------------------------------------------------------------------------------
;	cist stav
;	read status 
;--------------------------------------------------------------------------------------------------------
ReadDriveStat:	cbi	portd,3
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	in	R16,pinc
	sbi	portd,3
	nop
	nop
	nop
	nop
	ret
;--------------------------------------------------------------------------------------------------------
;	nastavit status registr
;	set status reg
;--------------------------------------------------------------------------------------------------------
SetDriveStatReg:	in	R17,ddrc
	ldi	R16,0b00000000
	out	ddrc,R16
	ldi	R16,0b11110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ret
;--------------------------------------------------------------------------------------------------------
;	nastavit datovy registr
;	set data reg
;--------------------------------------------------------------------------------------------------------
SetDriveDataReg:	ldi	R16,0b10000000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	ret
;--------------------------------------------------------------------------------------------------------
;	pulz na IOWR
;	IOWR puls
;--------------------------------------------------------------------------------------------------------
IOWRpulz:	nop
	nop
	cbi	portd,7
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	sbi	portd,7
	nop
	nop
	ret
;--------------------------------------------------------------------------------------------------------
;	zapise 1 sektor
;	write one sector
;--------------------------------------------------------------------------------------------------------
WRdriveSektor:	ldi	R18,0	;256 slov
WRsectorR:	ld	R16,Z+
	ld	R17,Z+
	out	porta,R16
	out	portc,R17
	cbi	portd,7
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	sbi	portd,7
	nop
	nop
	nop
	nop
	nop
	dec	R18
	brne	WRsectorR
	ret
;--------------------------------------------------------------------------------------------------------
;	precte 1 sektor
;	read one sector
;--------------------------------------------------------------------------------------------------------
RDdriveSektor:	ldi	R16,0b00000000
	out	ddra,R16
	out	ddrc,R16
	;
	ldi	R18,0	;256 slov
RDsectorR:	cbi	portd,3
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	in	R16,pina
	in	R17,pinc
	sbi	portd,3
	st	Z+,R16
	st	Z+,R17
	nop
	nop
	dec	R18
	brne	RDsectorR
	;
	ldi	R16,0b11111111
	out	ddra,R16
	out	ddrc,R16
	ret
;--------------------------------------------------------------------------------------------------------
;	identifikace zarizeni na IDE
;	read idedntification sector
;--------------------------------------------------------------------------------------------------------
IdentIDEdrive:	ldi	R16,0b11111111
	out	ddrc,R16
	;cekat na roztoceni disku
	call	WaitDriveRDY
	;command registr
	ldi	R16,0b11110000
	out	porta,R16
	cbi	portd,5
	sbi	portd,5
	;
	ldi	R16,0xEC	;identifikace
	out	portc,R16
	rcall	IOWRpulz
	;
	call	WaitDataReady
	call	SetDriveDataReg
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	ldi	R16,0b00000000
	out	ddra,R16
	out	ddrc,R16
	ldi	R16,0b11111111	;zvedaci odpory
	out	porta,R16
	out	portc,R16
	;
	ldi	R19,0	;256 slov do zasobniku
ReadIdentSector:	cbi	portd,3
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	in	R16,pina	;nacist high byte
	in	R17,pinc	;nacist low byte
	sbi	portd,3
	st	Z+,R16
	st	Z+,R17
	dec	R19
	brne	ReadIdentSector
	;
	ldi	R16,0b11111111
	out	ddrc,R16
	out	ddra,R16
	ret
	
;*******************************************************************************************
;	vetsina rutin pro praci s FAT 
;	FAT routines (this is NOT FAT12/16/32!!!, FAT means File Allocation Table)
;*******************************************************************************************
;--------------------------------------------------------------------------------------------------------
;	nacte alokacni tabulku do zasobniku
;	load FAT to the buffer
;--------------------------------------------------------------------------------------------------------
LoadIDE_AT:	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	;
	clr	R7	;LBA 0 - 1. cast tabulky
	clr	R8
	clr	R9
	clr	R10
	call	SetLBAadressRD
	call	WaitDataReady
	call	SetDriveDataReg
	call	RDdriveSektor
	inc	R7	;LBA 1 - 2. cast tabulky
	call	SetLBAadressRD
	call	WaitDataReady
	call	SetDriveDataReg
	call	RDdriveSektor
	ret	
;--------------------------------------------------------------------------------------------------------
;	ulozi alokacni tabulku ze zasobniku do prvnich 2 sektoru
;	store to the HDD
;--------------------------------------------------------------------------------------------------------
SaveIDE_AT:	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	;uvodni retezec do zasobniku (pro jistotu obnoveni)
	ldi	R16,'T'
	ldi	R17,'a'
	ldi	R18,'b'
	ldi	R19,'0'
	st	Z+,R16
	st	Z+,R17
	st	Z+,R18
	st	Z+,R19
	;ted zapis na HDD/CF
	sbiw	R30,4
	clr	R7	;LBA 0 - 1. cast
	clr	R8
	clr	R9
	clr	R10
	call	SetLBAadressWR
	call	WRdriveSektor
	inc	R7	; LBA 1 - 2. cast
	call	SetLBAadressWR
	call	WRdriveSektor
	ret
;--------------------------------------------------------------------------------------------------------
;	spocita skladby v tabulce, ulozi do R19, secte jejich celkovou velikost do R28:R29
;	count songs
;--------------------------------------------------------------------------------------------------------
CountSongsAT:	ldi	R30,low(OutputFIFO+4)	;ukazatel na prvni bunku tabulky
	ldi	R31,high(OutputFIFO+4)
	clr	R28
	clr	R29
	ldi	R19,255	;max 255 skladeb
	;
	ldi	R20,255	;pocet bunek
ATbPocitejSkl:	ld	R16,Z+
	ld	R17,Z+
	ld	R26,Z+
	ld	R27,Z+	;celkem 4 bunky
	sub	R27,R17
	sbc	R26,R16
	add	R28,R27
	adc	R29,R26
	;
	clr	R18
	cp	R16,R18
	cpc	R17,R18
	brne	ATbNeniPrazdna
	;prazdna bunka
	dec	R19	;ubrat jednu skladbu z pocitadla
	;dalsi bunka
ATbNeniPrazdna:	dec	R20
	brne	ATbPocitejSkl
	;spocitano - v R19 by mel byt pocet skladeb
	ret
;--------------------------------------------------------------------------------------------------------
;	vytvori prazdnou alokacni tabulku a identifikacni retezec "Tab0"
;	format
;--------------------------------------------------------------------------------------------------------
FormatAlTab:	;nejdriv vymazat zasobnik
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	;uvodni retezec do zasobniku
	ldi	R16,'T'
	ldi	R17,'a'
	ldi	R18,'b'
	ldi	R19,'0'
	st	Z+,R16
	st	Z+,R17
	st	Z+,R18
	st	Z+,R19
	;zbytek tabulky nulovat
	ldi	R16,0
	ldi	R19,255	;255x 4 byty
ClrStackFormat:	st	Z+,R16
	st	Z+,R16
	st	Z+,R16
	st	Z+,R16
	dec	R19
	brne	ClrStackFormat
	;ted zapis na HDD/CF
	ldi	R30,low(OutputFIFO)
	ldi	R31,high(OutputFIFO)
	clr	R7	;LBA 0 - 1. cast
	clr	R8
	clr	R9
	clr	R10
	call	SetLBAadressWR
	call	WRdriveSektor
	inc	R7	; LBA 1 - 2. cast
	call	SetLBAadressWR
	call	WRdriveSektor
	;vynulovat pocet skladeb a ukazatel skladby
	ldi	R16,0
	sts	SongN,R16
	sts	SongPTR,R16
	ret
	;smaze z alokacni tabulky udaj o skladbe (nahadi ho 0:0:0:0)
ErSongPTRsong:	ldi	R30,low(OutputFIFO+4)
	ldi	R31,high(OutputFIFO+4)	;prvni bunka
	lds	R16,SongPTR
	ldi	R17,0
	lsl	R16
	rol	R17
	lsl	R16
	rol	R17	;nasobit 4x
	add	R30,R16
	adc	R31,R17
	;
	clr	R16
	st	Z+,R16
	st	Z+,R16
	st	Z+,R16
	st	Z+,R16	;smazat bunku
	ret
	
;--------------------------------------------------------------------------------------------------------
;	vytvori mezeru v alokacni tabulce na pozici dane R21
;	this create gap in the FAT at R21 position
;--------------------------------------------------------------------------------------------------------
CreateZeroCell:	push	R30
	push	R31
	;
	ldi	R30,low(OutputFIFO+1016)
	ldi	R31,high(OutputFIFO+1016)	;ukazatel na bunku 254 (predposledni bunka)
	;
CreateZeroCell1:	ld	R0,Z+
	ld	R1,Z+
	ld	R2,Z+
	ld	R3,Z+	;nacist ji
	st	Z+,R0
	st	Z+,R1
	st	Z+,R2
	st	Z+,R3	;a ulozit o 4B dal
	;jedena bunka je posunuta a dalsi
	;ukazatel na zacatek dalsi
	sbiw	R30,12
	;dalsi bunka
	dec	R21	;pocet posunovanych bunek
	brne	CreateZeroCell1
	;ted by melo byt posunutych prave R19 bunek a mezera pripravena
	pop	R31
	pop	R30
	ret
;--------------------------------------------------------------------------------------------------------
;	odstrani 1. nulovou bunku z alokacni tabulky
;	this removes forst zero cell from the FAT
;--------------------------------------------------------------------------------------------------------
RemFreeCellAT:	ldi	R30,low(OutputFIFO+4)
	ldi	R31,high(OutputFIFO+4)	;prvni bunka
	clr	R0
	sts	OutputFIFO+1024,R0
	sts	OutputFIFO+1025,R0
	sts	OutputFIFO+1026,R0
	sts	OutputFIFO+1027,R0	;tahle bunka bude po odstraneni posledni
	;najit bunku
	ldi	R20,255
RemBatNNl:	ld	R16,Z+
	ld	R17,Z+
	ld	R18,Z+
	ld	R19,Z+	;nacte bunku
	clr	R0
	cp	R16,R0
	cpc	R17,R0
	cpc	R18,R0
	cpc	R19,R0
	brne	RemBatNN
	;nasla se nulova bunka
	rjmp	RemBatAN
RemBatNN:	dec	R20
	brne	RemBatNNl
	ldi	R20,1	;nestane se to, ale pro jistotu
	;odstranit ji
RemBatAN:	ld	R16,Z+
	ld	R17,Z+
	ld	R18,Z+
	ld	R19,Z+	;nacist bunku
	sbiw	R30,8	;vratit se na minulou bunku
	st	Z+,R16
	st	Z+,R17
	st	Z+,R18
	st	Z+,R19	;prepsat ji nactenou
	adiw	R30,4	;skocit na dalsi bunku
	dec	R20
	brne	RemBatAN	;opakovat do konce alokacni tabuky
	ret
	
;*******************************************************************************************
;	Rutiny zpozdeni
;	delays
;*******************************************************************************************	
	;nastavitelne zpozdeni (R16)
DelayLongVar:	clr	R0
	clr	R1
DelLvar:	dec	R1
	brne	DelLvar
	dec	R0
	brne	DelLvar
	dec	R16
	brne	DelLvar
	ret
	;nastavitelne kratke zpozdeni (R16)
DelayShortVar:	clr	R0
DelSvar:	dec	R0
	brne	DelSvar
	dec	R16
	brne	DelSvar
	ret

;*******************************************************************************************
;	tady zacina matematika
;	math
;*******************************************************************************************
;***********************************
;*	IntDiv  24b/24b   *
;***********************************
;Tato rutina neni Float knihovnou pouzita, nicmene se muze hodit.
;Vstup:	RegA2:RegA1:RegA0 - delenec - bude prepsan podilem
;	RegB2:RegB1:RegB0 - delitel - nezmeni se
;Vystup:	RegA2:RegA1:RegA0 - podil
;	RegA5:RegA4:RegA3 - zbytek

	.def	RegA0=R16
	.def	RegA1=R17
	.def	RegA2=R18
	.def	RegB0=R19
	.def	RegB1=R20
	.def	RegB2=R21
	.def	RegA3=R13
	.def	RegA4=R14
	.def	RegA5=R15
		
Div24u:	clr	RegA5	;smazat zbytek
	clr	RegA4
	sub	RegA3,RegA3	;a zaroven Carry

	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_1	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_1:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_2	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_2:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_3		 ;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_3:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_4	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_4:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_5	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_5:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_6	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_6:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_7	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_7:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_8	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_8:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_9	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_9:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_10		;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_10:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_11		;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_11:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_12	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_12:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_13	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_13:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_14	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_14:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_15	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_15:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_16	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_16:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_17	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_17:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_18	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_18:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_19	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_19:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_20	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_20:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_21	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_21:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_22	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_22:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_23	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_23:	rol	RegA0	;rotaci delenec do zbytku
	rol	RegA1
	rol	RegA2
	rol	RegA3
	rol	RegA4
	rol	RegA5
	sub	RegA3,RegB0	;pokusit se odecist
	sbc	RegA4,RegB1
	sbc	RegA5,RegB2
	brcc	Div24u_24	;vyslo to, nechat
	add	RegA3,RegB0	;nevyslo to, pricist zpet
	adc	RegA4,RegB1
	adc	RegA5,RegB2

Div24u_24:	rol	RegA0	;jeste dorotovat podil
	rol	RegA1
	rol	RegA2
	com	RegA0	;je v komplementovanem tvaru
	com	RegA1
	com	RegA2
	ret
	
;*************** HIGH SPEED DIVIDE ********************************
;*
;* Editor: Pavel Borovsky, Czech Republic, Europe 3.6.2000
;* mail: profesor@ti.cz
;* (from Ap.Note AVR200b )
;*
;* "div16u" - 16/16 Bit Unsigned Division
;*
;* This subroutine divides the two 16-bit numbers 
;* "dd8uH:dd8uL" (dividend) and "dv16uH:dv16uL" (divisor). 
;* The result is placed in "dres16uH:dres16uL" and the remainder in
;* "drem16uH:drem16uL".
;* 
;* NEW !!! Number of words	  :150 + return
;* NEW !!! Number of cycles  :126/134/142 (Min/Avg/Max)
;*              
;* Low registers used	:2 (drem16uL,drem16uH)
;* High registers used  :4 (dres16uL/dd16uL,dres16uH/dd16uH,dv16uL,dv16uH)
;*
;***************************************************************************
;***** Subroutine Register Variables
	.def	drem16uL=R14
	.def	drem16uH=R15
	.def	dres16uL=R16
	.def	dres16uH=R17
	.def	dd16uL=R16
	.def	dd16uH=R17
	.def	dv16uL=R18
	.def	dv16uH=R19
	
div16u:	clr	drem16uL	;clear remainder Low byte
	sub	drem16uH,drem16uH	;clear remainder High byte and carry
	rol	dd16uL	;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	
	brcc	d16u_2	;if result negative
	add	drem16uL,dv16uL	;restore remainder
	adc	drem16uH,dv16uH
d16u_2:	rol	dd16uL	;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_4	;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_4:	rol	dd16uL	;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_6		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_6:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_8		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_8:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_10		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_10:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_12		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_12:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_14		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_14:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_16		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_16:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_18		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_18:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_20		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_20:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_22		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_22:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_24		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_24:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_26		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_26:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_28		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_28:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_30		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_30:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	rol	drem16uL	;shift dividend into remainder
	rol	drem16uH
	sub	drem16uL,dv16uL	;remainder = remainder - divisor
	sbc	drem16uH,dv16uH	;
	brcc	d16u_32		;if result negative
	add	drem16uL,dv16uL	;    restore remainder
	adc	drem16uH,dv16uH
d16u_32:	rol	dd16uL		;shift left dividend
	rol	dd16uH
	com	dd16uL
	com	dd16uH
	ret
	
	;nasobeni 16x16
mul16u:	clr	R2
	mul	R23,R21		; ah * bh
	movw	R19:R18,R1:R0
	mul	R22,R20		; al * bl
	movw	R17:R16,R1:R0
	mul	R23,R20		; ah * bl
	add	R17,R0
	adc	R18,R1
	adc	R19,R2
	mul	R21,R22		; bh * al
	add	R17,R0
	adc	R18,R1
	adc	R19,R2
	ret

	;prevod z binarniho do dekadickeho tvaru pro 8bit cislo
bin2bcd8:	clr	R17
	clr	R16
bBCD8_1:	subi	R18,10
	brcs	bBCD8_2
	inc	R17
	subi	R17,10
	brcs	bBCD8_3
	inc	R16
	rjmp	bBCD8_1
bBCD8_3:	subi	R17,-10
	rjmp	bBCD8_1
bBCD8_2:	subi	R18,-10
	;na ASCII
	subi	R16,-48
	subi	R17,-48
	subi	R18,-48
	;nahradi nuly na zacatku mezerou	
	cpi	R16,48
	brne	DeNulovano
	ldi	R16,' '
	cpi	R17,48
	brne	DeNulovano
	ldi	R17,' '
DeNulovano:	ret

;***************************************************************************
;*
;* "bin2BCD16" - 16-bit Binary to BCD conversion
;*
;* This subroutine converts a 16-bit number (fbinH:fbinL) to a 5-digit 
;* packed BCD number represented by 3 bytes (tBCD2:tBCD1:tBCD0).
;* MSD of the 5-digit number is placed in the lowermost nibble of tBCD2.
;*  
;* Number of words	:25
;* Number of cycles	:751/768 (Min/Max)
;* Low registers used	:3 (tBCD0,tBCD1,tBCD2) 
;* High registers used  :4(fbinL,fbinH,cnt16a,tmp16a)	
;* Pointers used	:Z
;*
;***************************************************************************

;***** Subroutine Register Variables

	.equ	AtBCD0=13	;address of tBCD0
	.equ	AtBCD2=15	;address of tBCD1

	.def	tBCD0=r13	;BCD value digits 1 and 0
	.def	tBCD1=r14	;BCD value digits 3 and 2
	.def	tBCD2=r15	;BCD value digit 4
	.def	fbinL=r16	;binary value Low byte
	.def	fbinH=r17	;binary value High byte
	.def	cnt16a=r18	;loop counter
	.def	tmp16a=r19	;temporary value

bin2BCD16:	ldi	cnt16a,16	;Init loop counter	
	clr	tBCD2	;clear result (3 bytes)
	clr	tBCD1
	clr	tBCD0
	clr	ZH	;clear ZH (not needed for AT90Sxx0x)
bBCDx_1:	lsl	fbinL	;shift input value
	rol	fbinH	;through all bytes
	rol	tBCD0	;
	rol	tBCD1
	rol	tBCD2
	dec	cnt16a	;decrement loop counter
	brne	bBCDx_2	;if counter not zero
	ret		;return
bBCDx_2:	ldi	r30,AtBCD2+1	;Z points to result MSB + 1
bBCDx_3:	ld	tmp16a,-Z	;get (Z) with pre-decrement
	subi	tmp16a,-$03	;add 0x03
	sbrc	tmp16a,3	;if bit 3 not clear
	st	Z,tmp16a	;store back
	ld	tmp16a,Z	;get (Z)
	subi	tmp16a,-$30	;add 0x30
	sbrc	tmp16a,7	;if bit 7 not clear
	st	Z,tmp16a	;store back
	cpi	ZL,AtBCD0	;done all three?
	brne	bBCDx_3	;loop again if not
	rjmp	bBCDx_1
	
;*******************************************************************************************
;	veskere upravitelne texty
;	0xFF - zakonceni vypisu (nesmi chybet!!!) (line termination)
;	0x00 - zalomeni radku (pouze z 1. na 2. (opacne nefunguje)) (break line)
;*******************************************************************************************
TextNoDriveT:	.db	"Nebylo nalezeno",0x00,"zadne IDE zarizeni",0xFF
TextTabNeniT:	.db	"Chybi alokacni tab.",0x00,"Znovu/Format/bez ",0xFF
TextForUT:	.db	"Urcite formatovat ?",0x00,"Ano/Ne",0xFF
TextMSvolT:	.db	"Urcite mazat skladbu",0x00,"Ano/ne",0xFF
TextNST:	.db	"Nalezenych skladeb:",0x00,0xFF
TextMvolT:	.db	"Mazat",0x00,"Skladbu/Vse/Info/Nic",0xFF
TextRRNMT:	.db	"Nelze mazat skladbu",0x00,"hraje se z RAM",0xFF
TextNNSzCT:	.db	"Nejdriv nacist",0x00,"skladbu z COM portu",0xFF
TextSMPT:	.db	"Skladbu nelze ulozit",0x00,"max. 255 skladeb",0xFF
TextNUNPT:	.db	"Skladbu nelze ulozit",0x00,"nedostatek pameti",0xFF
TextSJVT:	.db	"Skladba ",0xFF
TextSMAT:	.db	" ma ",0xFF
TextNDT:	.db	"Nacitam data z ",0xFF
TextCNDT:	.db	"Cekam na data z USB",0xFF
TextUDT:	.db	"Ukladam data na ",0xFF
TextCFT:	.db	"CompactFlash ",0xFF
TextCFsT:	.db	"CF",0xFF
TextHDDT:	.db	"HDD ",0xFF
TextKBT:	.db	"kB",0xFF
TextMBT:	.db	"MB ",0xFF
TextVeT:	.db	" v ",0xFF
TextCHST:	.db	"CHS ",0xFF

	;english version
;TextNoDriveT:	.db	"IDE device not found",0xFF
;TextTabNeniT:	.db	"FAT not found ...",0x00,"Again/Format/no ",0xFF
;TextForUT:	.db	"Format disk?",0x00,"Yes/No",0xFF
;TextMSvolT:	.db	"Erase song?",0x00,"Yes/No",0xFF
;TextNST:	.db	"Founded songs:",0x00,0xFF
;TextMvolT:	.db	"Erase",0x00,"Song/All/Info/Nothing",0xFF
;TextRRNMT:	.db	"Can not erase song",0x00,"not saved yet",0xFF
;TextNNSzCT:	.db	"First load song",0x00,"from COM/USB",0xFF
;TextSMPT:	.db	"Can not save song",0x00,"max. 255 songs",0xFF
;TextNUNPT:	.db	"Can not save song",0x00,"not enough memory",0xFF
;TextSJVT:	.db	"Song ",0xFF
;TextSMAT:	.db	" len ",0xFF
;TextNDT:	.db	"Loading from ",0xFF
;TextCNDT:	.db	"Waiting for data ...",0xFF
;TextUDT:	.db	"Saving data to ",0xFF
;TextCFT:	.db	"CompactFlash ",0xFF
;TextCFsT:	.db	"CF",0xFF
;TextHDDT:	.db	"HDD ",0xFF
;TextKBT:	.db	"kB",0xFF
;TextMBT:	.db	"MB ",0xFF
;TextVeT:	.db	" in ",0xFF
;TextCHST:	.db	"CHS ",0xFF

